@@ -30,6 +30,8 @@ registers its _p_jar attribute.  TODO: explain adapter
 Subtransactions
 ---------------
 
+Note: Suntransactions are deprecated!  Use savepoint/rollback instead.
+
 A subtransaction applies the transaction notion recursively.  It
 allows a set of modifications within a transaction to be committed or
 aborted as a group.  A subtransaction is a strictly local activity;
@@ -54,8 +56,10 @@ methods and support a second argument to tpc_begin().
 
 The second argument to tpc_begin() indicates that a subtransaction
 commit is beginning (if it is true).  In a subtransaction, there is no
-tpc_vote() call.  (XXX: I don't have any idea why.)  The tpc_finish()
-or tpc_abort() call applies just to that subtransaction.
+tpc_vote() call, because sub-transactions don't need 2-phase commit.
+If a sub-transaction abort or commit fails, we can abort the outer
+transaction.  The tpc_finish() or tpc_abort() call applies just to
+that subtransaction.
 
 Once a resource manager is involved in a subtransaction, all
 subsequent transactions will be treated as subtransactions until
@@ -80,7 +84,7 @@ calls the following four methods on each resource manager; it calls
 tpc_begin() on each resource manager before calling commit() on any of
 them.
 
-    1. tpc_begin(txn, subtransaction=False)
+    1. tpc_begin(txn)
     2. commit(txn)
     3. tpc_vote(txn)
     4. tpc_finish(txn)
@@ -88,6 +92,8 @@ them.
 Subtransaction commit
 ---------------------
 
+Note: Subtransactions are deprecated!
+
 When a subtransaction commits, the protocol is different.
 
 1. tpc_begin() is passed a second argument, which indicates that a
@@ -98,6 +104,17 @@ Once a subtransaction has been committed, the top-level transaction
 commit will start with a commit_sub() call instead of a tpc_begin()
 call.
 
+Before-commit hook
+---------------
+
+Sometimes, applications want to execute some code when a transaction is
+committed.  For example, one might want to delay object indexing until a
+transaction commits, rather than indexing every time an object is changed.
+Or someone might want to check invariants only after a set of operations.  A
+pre-commit hook is available for such use cases, just use beforeCommitHook()
+passing it a callable and arguments.  The callable will be called with its
+arguments at the start of the commit (but not for substransaction commits).
+
 Error handling
 --------------
 
@@ -115,8 +132,7 @@ uncommitted object, including the object that failed in its commit(),
 call abort().
 
 Once uncommitted objects are aborted, tpc_abort() or abort_sub() is
-called on each resource manager.  abort_sub() is called if the
-resource manager was involved in a subtransaction.
+called on each resource manager.
 
 Synchronization
 ---------------
@@ -128,18 +144,19 @@ manager calls beforeCompletion() when it starts a top-level two-phase
 commit.  It calls afterCompletion() when a top-level transaction is
 committed or aborted.  The methods are passed the current Transaction
 as their only argument.
-
-XXX This code isn't tested.
-
 """
 
 import logging
 import sys
 import thread
 import warnings
+import weakref
 import traceback
 from cStringIO import StringIO
 
+from zope import interface
+from transaction import interfaces
+
 # Sigh.  In the maze of __init__.py's, ZODB.__init__.py takes 'get'
 # out of transaction.__init__.py, in order to stuff the 'get_transaction'
 # alias in __builtin__.  So here in _transaction.py, we can't import
@@ -168,6 +185,27 @@ class Status:
 
 class Transaction(object):
 
+    interface.implements(interfaces.ITransaction,
+                         interfaces.ITransactionDeprecated)
+
+
+    # Assign an index to each savepoint so we can invalidate later savepoints
+    # on rollback.  The first index assigned is 1, and it goes up by 1 each
+    # time.
+    _savepoint_index = 0
+
+    # If savepoints are used, keep a weak key dict of them.  This maps a
+    # savepoint to its index (see above).
+    _savepoint2index = None
+
+    # Remember the savepoint for the last subtransaction.
+    _subtransaction_savepoint = None
+
+    # Meta data.  ._extension is also metadata, but is initialized to an
+    # emtpy dict in __init__.
+    user = ""
+    description = ""
+
     def __init__(self, synchronizers=None, manager=None):
         self.status = Status.ACTIVE
         # List of resource managers, e.g. MultiObjectResourceAdapters.
@@ -190,39 +228,34 @@ class Transaction(object):
 
         # The user, description, and _extension attributes are accessed
         # directly by storages, leading underscore notwithstanding.
-        self.user = ""
-        self.description = ""
         self._extension = {}
 
         self.log = logging.getLogger("txn.%d" % thread.get_ident())
         self.log.debug("new transaction")
 
-        # _sub contains all of the resource managers involved in
-        # subtransactions.  It maps id(a resource manager) to the resource
-        # manager.
-        self._sub = {}
-        # _nonsub contains all the resource managers that do not support
-        # subtransactions that were involved in subtransaction commits.
-        self._nonsub = {}
-
         # If a commit fails, the traceback is saved in _failure_traceback.
         # If another attempt is made to commit, TransactionFailedError is
         # raised, incorporating this traceback.
         self._failure_traceback = None
 
+        # Holds (hook, args, kws) triples added by beforeCommitHook.
+        # TODO:  in Python 2.4, change to collections.deque; lists can be
+        # inefficient for FIFO access of this kind.
+        self._before_commit = []
+
     # Raise TransactionFailedError, due to commit()/join()/register()
     # getting called when the current transaction has already suffered
-    # a commit failure.
-    def _prior_commit_failed(self):
+    # a commit/savepoint failure.
+    def _prior_operation_failed(self):
         from ZODB.POSException import TransactionFailedError
         assert self._failure_traceback is not None
-        raise TransactionFailedError("commit() previously failed, "
-                "with this traceback:\n\n%s" %
+        raise TransactionFailedError("An operation previously failed, "
+                "with traceback:\n\n%s" %
                 self._failure_traceback.getvalue())
 
     def join(self, resource):
         if self.status is Status.COMMITFAILED:
-            self._prior_commit_failed() # doesn't return
+            self._prior_operation_failed() # doesn't return
 
         if self.status is not Status.ACTIVE:
             # TODO: Should it be possible to join a committing transaction?
@@ -233,9 +266,62 @@ class Transaction(object):
         # be better to use interfaces.  If this is a ZODB4-style
         # resource manager, it needs to be adapted, too.
         if myhasattr(resource, "prepare"):
+            # TODO: deprecate 3.6
             resource = DataManagerAdapter(resource)
         self._resources.append(resource)
 
+        if self._savepoint2index:
+            # A data manager has joined a transaction *after* a savepoint
+            # was created.  A couple of things are different in this case:
+            #
+            # 1. We need to add its savepoint to all previous savepoints.
+            # so that if they are rolled back, we roll this one back too.
+            #
+            # 2. We don't actually need to ask the data manager for a
+            # savepoint:  because it's just joining, we can just abort it to
+            # roll back to the current state, so we simply use an
+            # AbortSavepoint.
+            datamanager_savepoint = AbortSavepoint(resource, self)
+            for transaction_savepoint in self._savepoint2index.keys():
+                transaction_savepoint._savepoints.append(
+                    datamanager_savepoint)
+
+    def savepoint(self, optimistic=False):
+        if self.status is Status.COMMITFAILED:
+            self._prior_operation_failed() # doesn't return, it raises
+
+        try:
+            savepoint = Savepoint(self, optimistic, *self._resources)
+        except:
+            self._cleanup(self._resources)
+            self._saveCommitishError() # reraises!
+
+        if self._savepoint2index is None:
+            self._savepoint2index = weakref.WeakKeyDictionary()
+        self._savepoint_index += 1
+        self._savepoint2index[savepoint] = self._savepoint_index
+
+        return savepoint
+
+    # Remove and invalidate all savepoints we know about with an index
+    # larger than `savepoint`'s.  This is what's needed when a rollback
+    # _to_ `savepoint` is done.
+    def _remove_and_invalidate_after(self, savepoint):
+        savepoint2index = self._savepoint2index
+        index = savepoint2index[savepoint]
+        # use items() to make copy to avoid mutating while iterating
+        for savepoint, i in savepoint2index.items():
+            if i > index:
+                savepoint.transaction = None # invalidate
+                del savepoint2index[savepoint]
+
+    # Invalidate and forget about all savepoints.
+    def _invalidate_all_savepoints(self):
+        for savepoint in self._savepoint2index.keys():
+            savepoint.transaction = None # invalidate
+        self._savepoint2index.clear()
+
+
     def register(self, obj):
         # The old way of registering transaction participants.
         #
@@ -247,12 +333,11 @@ class Transaction(object):
         # commit protocol.
 
         manager = getattr(obj, "_p_jar", obj)
+        if manager is None:
+            raise ValueError("Register with no manager")
         adapter = self._adapters.get(manager)
         if adapter is None:
-            if myhasattr(manager, "commit_sub"):
-                adapter = MultiObjectResourceAdapterSub(manager)
-            else:
-                adapter = MultiObjectResourceAdapter(manager)
+            adapter = MultiObjectResourceAdapter(manager)
             adapter.objects.append(obj)
             self._adapters[manager] = adapter
             self.join(adapter)
@@ -262,89 +347,93 @@ class Transaction(object):
             assert id(obj) not in map(id, adapter.objects)
             adapter.objects.append(obj)
 
-            # In the presence of subtransactions, an existing adapter
-            # might be in _adapters but not in _resources.
-            if adapter not in self._resources:
-                self._resources.append(adapter)
-
     def begin(self):
         from ZODB.utils import deprecated36
 
         deprecated36("Transaction.begin() should no longer be used; use "
                       "the begin() method of a transaction manager.")
-        if (self._resources or
-              self._sub or
-              self._nonsub or
-              self._synchronizers):
+        if (self._resources or self._synchronizers):
             self.abort()
         # Else aborting wouldn't do anything, except if _manager is non-None,
         # in which case it would do nothing besides uselessly free() this
         # transaction.
 
-    def commit(self, subtransaction=False):
-        if self.status is Status.COMMITFAILED:
-            self._prior_commit_failed() # doesn't return
+    def commit(self, subtransaction=_marker, deprecation_wng=True):
+        if subtransaction is _marker:
+            subtransaction = 0
+        elif deprecation_wng:
+            from ZODB.utils import deprecated37
+            deprecated37("subtransactions are deprecated; use "
+                         "transaction.savepoint() instead of "
+                         "transaction.commit(1)")
 
-        if not subtransaction and self._sub and self._resources:
-            # This commit is for a top-level transaction that has
-            # previously committed subtransactions.  Do one last
-            # subtransaction commit to clear out the current objects,
-            # then commit all the subjars.
-            self.commit(True)
-
-        if not subtransaction:
-            self._synchronizers.map(lambda s: s.beforeCompletion(self))
-            self.status = Status.COMMITTING
-
-        try:
-            self._commitResources(subtransaction)
-        except:
-            self.status = Status.COMMITFAILED
-            # Save the traceback for TransactionFailedError.
-            ft = self._failure_traceback = StringIO()
-            t, v, tb = sys.exc_info()
-            # Record how we got into commit().
-            traceback.print_stack(sys._getframe(1), None, ft)
-            # Append the stack entries from here down to the exception.
-            traceback.print_tb(tb, None, ft)
-            # Append the exception type and value.
-            ft.writelines(traceback.format_exception_only(t, v))
-            raise t, v, tb
+        if self._savepoint2index:
+            self._invalidate_all_savepoints()
 
         if subtransaction:
-            self._resources = []
-        else:
-            self.status = Status.COMMITTED
-            if self._manager:
-                self._manager.free(self)
-            self._synchronizers.map(lambda s: s.afterCompletion(self))
-            self.log.debug("commit")
+            # TODO deprecate subtransactions
+            self._subtransaction_savepoint = self.savepoint(1)
+            return
 
-    def _commitResources(self, subtransaction):
+        if self.status is Status.COMMITFAILED:
+            self._prior_operation_failed() # doesn't return
+
+        self._callBeforeCommitHooks()
+
+        self._synchronizers.map(lambda s: s.beforeCompletion(self))
+        self.status = Status.COMMITTING
+
+        try:
+            self._commitResources()
+        except:
+            self._saveCommitishError() # This raises!
+
+        self.status = Status.COMMITTED
+        if self._manager:
+            self._manager.free(self)
+        self._synchronizers.map(lambda s: s.afterCompletion(self))
+        self.log.debug("commit")
+
+    def _saveCommitishError(self):
+        self.status = Status.COMMITFAILED
+        # Save the traceback for TransactionFailedError.
+        ft = self._failure_traceback = StringIO()
+        t, v, tb = sys.exc_info()
+        # Record how we got into commit().
+        traceback.print_stack(sys._getframe(1), None, ft)
+        # Append the stack entries from here down to the exception.
+        traceback.print_tb(tb, None, ft)
+        # Append the exception type and value.
+        ft.writelines(traceback.format_exception_only(t, v))
+        raise t, v, tb
+
+    def getBeforeCommitHooks(self):
+        return iter(self._before_commit)
+
+    def beforeCommitHook(self, hook, *args, **kws):
+        self._before_commit.append((hook, args, kws))
+
+    def _callBeforeCommitHooks(self):
+        # Call all hooks registered, allowing further registrations
+        # during processing.
+        while self._before_commit:
+            hook, args, kws = self._before_commit.pop(0)
+            hook(*args, **kws)
+
+    def _commitResources(self):
         # Execute the two-phase commit protocol.
 
-        L = self._getResourceManagers(subtransaction)
+        L = list(self._resources)
+        L.sort(rm_cmp)
         try:
             for rm in L:
-                # If you pass subtransaction=True to tpc_begin(), it
-                # will create a temporary storage for the duration of
-                # the transaction.  To signal that the top-level
-                # transaction is committing, you must then call
-                # commit_sub().
-                if not subtransaction and id(rm) in self._sub:
-                    del self._sub[id(rm)]
-                    rm.commit_sub(self)
-                else:
-                    rm.tpc_begin(self, subtransaction)
+                rm.tpc_begin(self)
             for rm in L:
                 rm.commit(self)
                 self.log.debug("commit %r" % rm)
-            if not subtransaction:
-                # Not sure why, but it is intentional that you do not
-                # call tpc_vote() for subtransaction commits.
-                for rm in L:
-                    rm.tpc_vote(self)
-                    self._voted[id(rm)] = True
+            for rm in L:
+                rm.tpc_vote(self)
+                self._voted[id(rm)] = True
 
             try:
                 for rm in L:
@@ -364,8 +453,7 @@ class Transaction(object):
             try:
                 self._cleanup(L)
             finally:
-                if not subtransaction:
-                    self._synchronizers.map(lambda s: s.afterCompletion(self))
+                self._synchronizers.map(lambda s: s.afterCompletion(self))
             raise t, v, tb
 
     def _cleanup(self, L):
@@ -378,68 +466,42 @@ class Transaction(object):
                     self.log.error("Error in abort() on manager %s",
                                    rm, exc_info=sys.exc_info())
         for rm in L:
-            if id(rm) in self._sub:
-                try:
-                    rm.abort_sub(self)
-                except Exception:
-                    self.log.error("Error in abort_sub() on manager %s",
-                                   rm, exc_info=sys.exc_info())
-            else:
-                try:
-                    rm.tpc_abort(self)
-                except Exception:
-                    self.log.error("Error in tpc_abort() on manager %s",
-                                   rm, exc_info=sys.exc_info())
+            try:
+                rm.tpc_abort(self)
+            except Exception:
+                self.log.error("Error in tpc_abort() on manager %s",
+                               rm, exc_info=sys.exc_info())
+
+    def abort(self, subtransaction=_marker, deprecation_wng=True):
+        if subtransaction is _marker:
+            subtransaction = 0
+        elif deprecation_wng:
+            from ZODB.utils import deprecated37
+            deprecated37("subtransactions are deprecated; use "
+                         "sp.rollback() instead of "
+                         "transaction.abort(1), where `sp` is the "
+                         "corresponding savepoint captured earlier")
 
-    def _getResourceManagers(self, subtransaction):
-        L = []
         if subtransaction:
-            # If we are in a subtransaction, make sure all resource
-            # managers are placed in either _sub or _nonsub.  When
-            # the top-level transaction commits, we need to merge
-            # these back into the resource set.
+            # TODO deprecate subtransactions.
+            if not self._subtransaction_savepoint:
+                raise interfaces.InvalidSavepointRollbackError
+            if self._subtransaction_savepoint.valid:
+                self._subtransaction_savepoint.rollback()
+                # We're supposed to be able to call abort(1) multiple
+                # times without additional effect, so mark the subtxn
+                # savepoint invalid now.
+                self._subtransaction_savepoint.transaction = None
+                assert not self._subtransaction_savepoint.valid
+            return
 
-            # If a data manager doesn't support sub-transactions, we
-            # don't do anything with it now.  (That's somewhat okay,
-            # because subtransactions are mostly just an
-            # optimization.)  Save it until the top-level transaction
-            # commits.
+        if self._savepoint2index:
+            self._invalidate_all_savepoints()
 
-            for rm in self._resources:
-                if myhasattr(rm, "commit_sub"):
-                    self._sub[id(rm)] = rm
-                    L.append(rm)
-                else:
-                    self._nonsub[id(rm)] = rm
-        else:
-            if self._sub or self._nonsub:
-                # Merge all of _sub, _nonsub, and _resources.
-                d = dict(self._sub)
-                d.update(self._nonsub)
-                # TODO: I think _sub and _nonsub are disjoint, and that
-                #       _resources is empty.  If so, we can simplify this code.
-                assert len(d) == len(self._sub) + len(self._nonsub)
-                assert not self._resources
-                for rm in self._resources:
-                    d[id(rm)] = rm
-                L = d.values()
-            else:
-                L = list(self._resources)
-
-        L.sort(rm_cmp)
-        return L
-
-    def abort(self, subtransaction=False):
-        if not subtransaction:
-            self._synchronizers.map(lambda s: s.beforeCompletion(self))
-
-        if subtransaction and self._nonsub:
-            from ZODB.POSException import TransactionError
-            raise TransactionError("Resource manager does not support "
-                                   "subtransaction abort")
+        self._synchronizers.map(lambda s: s.beforeCompletion(self))
 
         tb = None
-        for rm in self._resources + self._nonsub.values():
+        for rm in self._resources:
             try:
                 rm.abort(self)
             except:
@@ -448,21 +510,12 @@ class Transaction(object):
                 self.log.error("Failed to abort resource manager: %s",
                                rm, exc_info=sys.exc_info())
 
-        if not subtransaction:
-            for rm in self._sub.values():
-                try:
-                    rm.abort_sub(self)
-                except:
-                    if tb is None:
-                        t, v, tb = sys.exc_info()
-                    self.log.error("Failed to abort_sub resource manager: %s",
-                                   rm, exc_info=sys.exc_info())
+        if self._manager:
+            self._manager.free(self)
 
-        if not subtransaction:
-            if self._manager:
-                self._manager.free(self)
-            self._synchronizers.map(lambda s: s.afterCompletion(self))
-            self.log.debug("abort")
+        self._synchronizers.map(lambda s: s.afterCompletion(self))
+
+        self.log.debug("abort")
 
         if tb is not None:
             raise t, v, tb
@@ -502,8 +555,8 @@ class MultiObjectResourceAdapter(object):
     def sortKey(self):
         return self.manager.sortKey()
 
-    def tpc_begin(self, txn, sub=False):
-        self.manager.tpc_begin(txn, sub)
+    def tpc_begin(self, txn):
+        self.manager.tpc_begin(txn)
 
     def tpc_finish(self, txn):
         self.manager.tpc_finish(txn)
@@ -534,25 +587,6 @@ class MultiObjectResourceAdapter(object):
         if tb is not None:
             raise t, v, tb
 
-class MultiObjectResourceAdapterSub(MultiObjectResourceAdapter):
-    """Adapt resource managers that participate in subtransactions."""
-
-    def commit_sub(self, txn):
-        self.manager.commit_sub(txn)
-
-    def abort_sub(self, txn):
-        self.manager.abort_sub(txn)
-
-    def tpc_begin(self, txn, sub=False):
-        self.manager.tpc_begin(txn, sub)
-        self.sub = sub
-
-    def tpc_finish(self, txn):
-        self.manager.tpc_finish(txn)
-        if self.sub:
-            self.objects = []
-
-
 def rm_cmp(rm1, rm2):
     return cmp(rm1.sortKey(), rm2.sortKey())
 
@@ -572,6 +606,7 @@ def object_hint(o):
         oid = oid_repr(oid)
     return "%s oid=%s" % (klass, oid)
 
+# TODO: deprecate for 3.6.
 class DataManagerAdapter(object):
     """Adapt zodb 4-style data managers to zodb3 style
 
@@ -586,50 +621,86 @@ class DataManagerAdapter(object):
 
     def __init__(self, datamanager):
         self._datamanager = datamanager
-        self._rollback = None
 
     # TODO: I'm not sure why commit() doesn't do anything
 
     def commit(self, transaction):
+        # We don't do anything here because ZODB4-style data managers
+        # didn't have a separate commit step
         pass
 
     def abort(self, transaction):
-
-        # We need to discard any changes since the last save point, or all
-        # changes
-
-        if self._rollback is None:
-            # No previous savepoint, so just abort
-            self._datamanager.abort(transaction)
-        else:
-            self._rollback()
-
-    def abort_sub(self, transaction):
         self._datamanager.abort(transaction)
 
-    def commit_sub(self, transaction):
-        # Nothing to do wrt data, be we begin 2pc for the top-level
-        # trans
-        self._sub = False
-
-    def tpc_begin(self, transaction, subtransaction=False):
-        self._sub = subtransaction
+    def tpc_begin(self, transaction):
+        # We don't do anything here because ZODB4-style data managers
+        # didn't have a separate tpc_begin step
+        pass
 
     def tpc_abort(self, transaction):
-        if self._sub:
-            self.abort(self, transaction)
-        else:
-            self._datamanager.abort(transaction)
+        self._datamanager.abort(transaction)
 
     def tpc_finish(self, transaction):
-        if self._sub:
-            self._rollback = self._datamanager.savepoint(transaction).rollback
-        else:
-            self._datamanager.commit(transaction)
+        self._datamanager.commit(transaction)
 
     def tpc_vote(self, transaction):
-        if not self._sub:
-            self._datamanager.prepare(transaction)
+        self._datamanager.prepare(transaction)
 
     def sortKey(self):
         return self._datamanager.sortKey()
+
+class Savepoint:
+    """Transaction savepoint.
+
+    Transaction savepoints coordinate savepoints for data managers
+    participating in a transaction.
+    """
+    interface.implements(interfaces.ISavepoint)
+
+    valid = property(lambda self: self.transaction is not None)
+
+    def __init__(self, transaction, optimistic, *resources):
+        self.transaction = transaction
+        self._savepoints = savepoints = []
+
+        for datamanager in resources:
+            try:
+                savepoint = datamanager.savepoint
+            except AttributeError:
+                if not optimistic:
+                    raise TypeError("Savepoints unsupported", datamanager)
+                savepoint = NoRollbackSavepoint(datamanager)
+            else:
+                savepoint = savepoint()
+
+            savepoints.append(savepoint)
+
+    def rollback(self):
+        transaction = self.transaction
+        if transaction is None:
+            raise interfaces.InvalidSavepointRollbackError
+        transaction._remove_and_invalidate_after(self)
+
+        try:
+            for savepoint in self._savepoints:
+                savepoint.rollback()
+        except:
+            # Mark the transaction as failed.
+            transaction._saveCommitishError() # reraises!
+
+class AbortSavepoint:
+
+    def __init__(self, datamanager, transaction):
+        self.datamanager = datamanager
+        self.transaction = transaction
+
+    def rollback(self):
+        self.datamanager.abort(self.transaction)
+
+class NoRollbackSavepoint:
+
+    def __init__(self, datamanager):
+        self.datamanager = datamanager
+
+    def rollback(self):
+        raise TypeError("Savepoints unsupported", self.datamanager)
