@@ -18,11 +18,25 @@ $Id: metaconfigure.py 26889 2004-08-04 04:00:36Z pruggera $
 __docformat__ = 'restructuredtext'
 from zope.interface import implements
 from zope.app.component.metaconfigure import utility
+
+from zope.app.apidoc import classregistry
 from zope.app.apidoc.codemodule.interfaces import IAPIDocRootModule
 
+
 class RootModule(str):
     implements(IAPIDocRootModule)
 
 def rootModule(_context, module):
     """Register a new module as a root module for the class browser."""
     utility(_context, IAPIDocRootModule, RootModule(module), name=module)
+
+
+def setModuleImport(flag):
+    classregistry.__import_unknown_modules__ = flag
+
+def moduleImport(_context, allow):
+    """Set the __import_unknown_modules__ flag"""
+    return _context.action(
+        ('apidoc', '__import_unknown_modules__'),
+        setModuleImport,
+        (allow, ))
