@@ -13,7 +13,7 @@
 ##############################################################################
 """Component architecture related 'zope' ZCML namespace directive interfaces
 
-$Id: metadirectives.py 29338 2005-02-28 00:16:22Z srichter $
+$Id: metadirectives.py 38047 2005-08-23 17:00:30Z frerich $
 """
 __docformat__ = 'restructuredtext'
 
@@ -176,7 +176,7 @@ class IAdapterDirective(zope.interface.Interface):
 
     provides = zope.configuration.fields.GlobalInterface(
         title=_("Interface the component provides"),
-        description=_("This attribute specifes the interface the adapter"
+        description=_("This attribute specifies the interface the adapter"
                       " instance must provide."),
         required=False,
         )
@@ -219,6 +219,17 @@ class IAdapterDirective(zope.interface.Interface):
         default=False,
         )
 
+    locate = zope.configuration.fields.Bool(
+        title=_("Locate"),
+        description=_("""Make the adapter a locatable adapter
+
+        Located adapter should be used if a non-public permission
+        is used.
+        """),
+        required=False,
+        default=False,
+        )
+
 class ISubscriberDirective(zope.interface.Interface):
     """
     Register a subscriber
@@ -238,7 +249,7 @@ class ISubscriberDirective(zope.interface.Interface):
 
     provides = zope.configuration.fields.GlobalInterface(
         title=_("Interface the component provides"),
-        description=_("This attribute specifes the interface the adapter"
+        description=_("This attribute specifies the interface the adapter"
                       " instance must provide."),
         required=False,
         )
@@ -273,6 +284,17 @@ class ISubscriberDirective(zope.interface.Interface):
         default=False,
         )
 
+    locate = zope.configuration.fields.Bool(
+        title=_("Locate"),
+        description=_("""Make the subscriber a locatable subscriber
+
+        Located subscribers should be used if a non-public permission
+        is used.
+        """),
+        required=False,
+        default=False,
+        )
+
 class IUtilityDirective(IBasicComponentInformation):
     """Register a utility."""
 
@@ -440,7 +462,7 @@ class IRequireSubdirective(zope.interface.Interface):
         description=_("""
         This argument says that this content class should be configured in the
         same way the specified class' security is. If this argument is
-        specifed, no other argument can be used."""),
+        specified, no other argument can be used."""),
         required=False,
         )
     
