@@ -11,9 +11,9 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Classes to support implenting `IContained`
+"""Classes to support implementing `IContained`
 
-$Id: contained.py 29246 2005-02-22 11:28:03Z gintautasm $
+$Id: contained.py 37532 2005-07-28 16:26:49Z jim $
 """
 __docformat__ = 'restructuredtext'
 
@@ -143,11 +143,12 @@ def dispatchToSublocations(object, event):
        Now we'll register it:
 
          >>> from zope.app.testing import ztapi
-         >>> ztapi.handle([None, IObjectMovedEvent], handler)
+         >>> ztapi.subscribe([None, IObjectMovedEvent], None, handler)
 
        We also register our dispatcher:
 
-         >>> ztapi.handle([None, IObjectMovedEvent], dispatchToSublocations)
+         >>> ztapi.subscribe([None, IObjectMovedEvent], None,
+         ...                 dispatchToSublocations)
 
        We can then call the dispatcher for the root object:
 
@@ -377,11 +378,11 @@ def setitem(container, setitemf, name, object):
     >>> from zope.app.container.interfaces import IObjectMovedEvent
     >>> from zope.app.testing import ztapi
 
-    >>> ztapi.handle([IItem, IObjectAddedEvent],
-    ...              lambda obj, event: obj.setAdded(event))
+    >>> ztapi.subscribe([IItem, IObjectAddedEvent], None,
+    ...                 lambda obj, event: obj.setAdded(event))
 
-    >>> ztapi.handle([IItem, IObjectMovedEvent],
-    ...              lambda obj, event: obj.setMoved(event))
+    >>> ztapi.subscribe([IItem, IObjectMovedEvent], None,
+    ...                 lambda obj, event: obj.setMoved(event))
 
     >>> item = Item()
 
@@ -745,7 +746,7 @@ class NameChooser(object):
             i += 1
             n = name + u'-' + unicode(i) + suffix
 
-        # Make sure tha name is valid.  We may have started with something bad.
+        # Make sure the name is valid.  We may have started with something bad.
         self.checkName(n, object)
 
         return n
@@ -836,6 +837,9 @@ class ContainedProxyClassProvides(zope.interface.declarations.ClassProvides):
 
 class ContainedProxy(ContainedProxyBase):
 
+    # Prevent proxies from having their own instance dictionaries:
+    __slots__ = ()
+
     __safe_for_unpickling__ = True
 
     zope.interface.implements(IContained)
