@@ -13,14 +13,14 @@
 ##############################################################################
 """WebDAV ``PROPFIND`` HTTP verb implementation
 
-$Id: test_propfind.py 29143 2005-02-14 22:43:16Z srichter $
+$Id: test_propfind.py 30384 2005-05-18 16:43:12Z mkerrin $
 """
 __docformat__ = 'restructuredtext'
 from StringIO import StringIO
 from unittest import TestCase, TestSuite, main, makeSuite
 from datetime import datetime
 
-from zope.interface import Interface,  directlyProvides
+from zope.interface import Interface,  directlyProvides, implements
 from zope.publisher.interfaces.http import IHTTPRequest
 
 from zope.pagetemplate.tests.util import normalize_xml
@@ -39,14 +39,21 @@ from zope.app.dublincore.interfaces import IZopeDublinCore
 from zope.app.dublincore.annotatableadapter import ZDCAnnotatableAdapter
 from zope.app.annotation.interfaces import IAnnotatable, IAnnotations
 from zope.app.annotation.attribute import AttributeAnnotations
+from zope.app.size.interfaces import ISized
+from zope.app.file.interfaces import IFile
 
 from zope.app.dav import propfind
 from zope.app.dav.interfaces import IDAVSchema
 from zope.app.dav.interfaces import IDAVNamespace
 from zope.app.dav.interfaces import IDAVWidget
-from zope.app.dav.widget import TextDAVWidget, SequenceDAVWidget
+from zope.app.dav.interfaces import IXMLDAVWidget
+from zope.app.dav.interfaces import IXMLText
+from zope.app.dav.widget import TextDAVWidget, SequenceDAVWidget, \
+     XMLDAVWidget
+from zope.app.dav.interfaces import IXMLText
 from zope.app.dav.opaquenamespaces import DAVOpaqueNamespacesAdapter
 from zope.app.dav.opaquenamespaces import IDAVOpaqueNamespaces
+from zope.app.dav.adapter import DAVSchemaAdapter
 
 from unitfixtures import File, Folder, FooZPT
 
@@ -80,6 +87,18 @@ def _createRequest(body=None, headers=None, skip_headers=None):
     request = TestRequest(StringIO(body), StringIO(), _environ)
     return request
 
+class FileSized(object):
+    implements(ISized)
+
+    def __init__(self, context):
+        self.context = context
+
+    def sizeForSorting(self):
+        return 'byte', len(self.context.data)
+
+    def sizeForDisplay(self):
+        return 'big'
+
 class TestPlacefulPROPFIND(PlacefulSetup, TestCase):
 
     def setUp(self):
@@ -103,11 +122,15 @@ class TestPlacefulPROPFIND(PlacefulSetup, TestCase):
         ztapi.browserViewProviding(ITextLine, TextDAVWidget, IDAVWidget)
         ztapi.browserViewProviding(IDatetime, TextDAVWidget, IDAVWidget)
         ztapi.browserViewProviding(ISequence, SequenceDAVWidget, IDAVWidget)
+        ztapi.browserViewProviding(IXMLText, XMLDAVWidget, IXMLDAVWidget)
         ztapi.provideAdapter(IAnnotatable, IAnnotations, AttributeAnnotations)
         ztapi.provideAdapter(IAnnotatable, IZopeDublinCore,
                              ZDCAnnotatableAdapter)
         ztapi.provideAdapter(IAnnotatable, IDAVOpaqueNamespaces,
                              DAVOpaqueNamespacesAdapter)
+        ztapi.provideAdapter(None, IDAVSchema,
+                             DAVSchemaAdapter)
+        ztapi.provideAdapter(IFile, ISized, FileSized)
         sm = zapi.getGlobalSiteManager()
         directlyProvides(IDAVSchema, IDAVNamespace)
         sm.provideUtility(IDAVNamespace, IDAVSchema, 'DAV:')
@@ -216,11 +239,14 @@ class TestPlacefulPROPFIND(PlacefulSetup, TestCase):
         # Check HTTP Response
         self.assertEqual(request.response.getStatus(), 400)
         self.assertEqual(pfind.getDepth(), 'full')
-        
+
     def _checkPropfind(self, obj, req, expect, depth='0', resp=None):
-        body = '''<?xml version="1.0" ?>
-        <propfind xmlns="DAV:">%s</propfind>
-        ''' % req
+        if req:
+            body = '''<?xml version="1.0" ?>
+            <propfind xmlns="DAV:">%s</propfind>
+            ''' % req
+        else:
+            body = ''
         request = _createRequest(body=body, headers={
             'Content-type': 'text/xml', 'Depth': depth})
         resource_url = zapi.absoluteURL(obj, request)
@@ -244,7 +270,62 @@ class TestPlacefulPROPFIND(PlacefulSetup, TestCase):
         s1 = normalize_xml(request.response._body)
         s2 = normalize_xml(expect)
         self.assertEqual(s1, s2)
-        
+
+    def test_resourcetype(self):
+        file = self.file
+        folder = traverse(self.rootFolder, 'folder')
+        req = '''<prop>
+        <resourcetype/>
+        </prop>
+        '''
+        expect_file = '''<prop>
+        <resourcetype></resourcetype>
+        </prop>
+        '''
+        expect_folder = '''<prop>
+        <resourcetype><collection/></resourcetype>
+        </prop>
+        '''
+        self._checkPropfind(file, req, expect_file)
+        self._checkPropfind(folder, req, expect_folder)
+
+    def test_getcontentlength(self):
+        file = self.file
+        folder = traverse(self.rootFolder, 'folder')
+        req = '''<prop>
+        <getcontentlength/>
+        </prop>
+        '''
+        expected_file = '''<prop>
+        <getcontentlength>%d</getcontentlength>
+        </prop>
+        ''' % len(file.data)
+        expected_folder = '''<prop>
+        <getcontentlength></getcontentlength>
+        </prop>
+        '''
+        self._checkPropfind(file, req, expected_file)
+        self._checkPropfind(folder, req, expected_folder)
+
+    def test_getcontenttype(self):
+        root = self.rootFolder
+        file = self.file
+        folder = traverse(root, 'folder')
+        req = '''<prop>
+        <getcontenttype/>
+        </prop>
+        '''
+        expected_file = '''<prop>
+        <getcontenttype>text/plain</getcontenttype>
+        </prop>
+        '''
+        expected_folder = '''<prop>
+        <getcontenttype></getcontenttype>
+        </prop>
+        '''
+        self._checkPropfind(file, req, expected_file)
+        self._checkPropfind(folder, req, expected_folder)
+
     def test_davpropdctitle(self):
         root = self.rootFolder
         zpt = traverse(root, 'zpt')
@@ -253,11 +334,11 @@ class TestPlacefulPROPFIND(PlacefulSetup, TestCase):
         req = '''<prop xmlns:DC="http://www.purl.org/dc/1.1">
         <DC:title />
         </prop>'''
-        
+
         expect = '''<prop xmlns:a0="http://www.purl.org/dc/1.1">
         <title xmlns="a0">Test Title</title></prop>'''
         self._checkPropfind(zpt, req, expect)
-        
+
     def test_davpropdccreated(self):
         root = self.rootFolder
         zpt = traverse(root, 'zpt')
@@ -369,7 +450,39 @@ class TestPlacefulPROPFIND(PlacefulSetup, TestCase):
         resp = '''<?xml version="1.0" encoding="utf-8"?>
         <multistatus xmlns="DAV:">%s</multistatus>'''
         self._checkPropfind(folder, req, expect, depth='infinity', resp=resp)
-        
+
+    def test_davemptybodyallpropzptdepth0(self):
+        # RFC 2518, Section 8.1: A client may choose not to submit a
+        # request body.  An empty PROPFIND request body MUST be
+        # treated as a request for the names and values of all
+        # properties.
+
+        root = self.rootFolder
+        zpt = traverse(root, 'zpt')
+        dc = IZopeDublinCore(zpt)
+        dc.created = now = datetime.utcnow()
+
+        req = ''
+        expect = ''
+        props = getFieldNamesInOrder(IZopeDublinCore)
+        pvalues = {'created': '%s+00:00' % now}
+        for p in props:
+            if pvalues.has_key(p):
+                expect += '<%s xmlns="a0">%s</%s>' % (p, pvalues[p], p)
+            else:
+                expect += '<%s xmlns="a0"></%s>' % (p, p)
+        props = getFieldNamesInOrder(IDAVSchema)
+        pvalues = {'displayname':'zpt',
+                   'creationdate':now.strftime('%Y-%m-%d %TZ')}
+        for p in props:
+            if pvalues.has_key(p):
+                expect += '<%s>%s</%s>' % (p, pvalues[p], p)
+            else:
+                expect += '<%s></%s>' % (p, p)
+        expect = '''<prop xmlns:a0="http://www.purl.org/dc/1.1">
+        %s</prop>''' % expect
+        self._checkPropfind(zpt, req, expect)
+
     def test_propfind_opaque_simple(self):
         root = self.rootFolder
         zpt = traverse(root, 'zpt')
@@ -385,7 +498,7 @@ class TestPlacefulPROPFIND(PlacefulSetup, TestCase):
         root = self.rootFolder
         zpt = traverse(root, 'zpt')
         oprops = IDAVOpaqueNamespaces(zpt)
-        oprops[u'http://foo/bar'] = {u'egg': 
+        oprops[u'http://foo/bar'] = {u'egg':
             '<egg xmlns:bacon="http://bacon">\n'
             '  <bacon:pork>crispy</bacon:pork>\n'
             '</egg>\n'}
