@@ -13,9 +13,11 @@
 ##############################################################################
 """Bootstrap code for error reporting utility.
 
-$Id: bootstrap.py 28023 2004-10-12 18:11:29Z anguenot $
+$Id: bootstrap.py 30460 2005-05-20 10:35:18Z hdima $
 """
 
+import transaction
+
 from zope.app.appsetup.bootstrap import ensureUtility, getInformationFromEvent
 
 from zope.app.error.error import RootErrorReportingUtility
@@ -29,8 +31,8 @@ def bootStrapSubscriber(event):
 
     db, connection, root, root_folder = getInformationFromEvent(event)
 
-    ensureUtility(root_folder, IErrorReportingUtility, 'ErrorReporting',
+    ensureUtility(root_folder, IErrorReportingUtility, '',
                   RootErrorReportingUtility, copy_to_zlog=False)
 
-    get_transaction().commit()
+    transaction.commit()
     connection.close()
