@@ -13,7 +13,7 @@
 ##############################################################################
 """'browser' namespace directive tests
 
-$Id: test_directives.py 29338 2005-02-28 00:16:22Z srichter $
+$Id: test_directives.py 29897 2005-04-07 14:26:38Z philikon $
 """
 import os
 import unittest
@@ -115,6 +115,28 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         XMLConfig('meta.zcml', zope.app.publisher.browser)()
         ztapi.provideAdapter(None, ITraversable, DefaultTraversable)
 
+    def testLayer(self):
+        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
+                         None)
+        xmlconfig(StringIO(
+            template % '<browser:layer name="testlayer" />'
+            ))
+        testlayer = zapi.getUtility(ILayer, "testlayer")
+        import zope.app.layers
+        self.assert_(zope.app.layers.testlayer is testlayer)
+
+    def testSkin(self):
+        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
+                         None)
+        xmlconfig(StringIO(template % (
+            '''
+            <browser:layer name="default" />
+            <browser:skin name="testskin" layers="default" />
+            '''
+            )))
+        testskin = zapi.getUtility(ISkin, "testskin")
+        import zope.app.skins
+        self.assert_(zope.app.skins.testskin is testskin)
 
     def testPage(self):
         self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
@@ -156,8 +178,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
                 />
             ''' % testtemplate
             )))
-        test_menu = zapi.getUtility(IMenuItemType, 'test_menu')
-        menuItem = getFirstMenuItem(test_menu, ob, TestRequest())
+        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
         v = zapi.queryMultiAdapter((ob, request), name='test')
@@ -184,8 +205,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % testtemplate
             )))
 
-        test_menu = zapi.getUtility(IMenuItemType, 'test_menu')
-        menuItem = getFirstMenuItem(test_menu, ob, TestRequest())
+        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
         v = zapi.queryMultiAdapter((ob, request), name='test')
@@ -214,8 +234,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % testtemplate
             )))
 
-        test_menu = zapi.getUtility(IMenuItemType, 'test_menu')
-        menuItem = getFirstMenuItem(test_menu, ob, TestRequest())
+        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
         v = zapi.queryMultiAdapter((ob, request), name='test')
@@ -246,8 +265,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % testtemplate
             )))
 
-        test_menu = zapi.getUtility(IMenuItemType, 'test_menu')
-        menuItem = getFirstMenuItem(test_menu, ob, TestRequest())
+        menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
         v = zapi.queryMultiAdapter((ob, request), name='test')
@@ -1139,6 +1157,33 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
 
         self.assert_(isinstance(v, V1))
 
+    def testMenuItemNeedsFor(self):
+	# <browser:menuItem> directive fails if no 'for' argument was provided
+	from zope.configuration.exceptions import ConfigurationError
+        self.assertRaises(ConfigurationError, xmlconfig, StringIO(template %
+            '''
+            <browser:menu
+                id="test_menu" title="Test menu" />
+	    <browser:menuItem
+	        title="Test Entry"
+	        menu="test_menu"
+		action="@@test"
+		/>
+            '''
+            ))
+
+	# it works, when the argument is there and a valid interface
+	xmlconfig(StringIO(template %
+            '''
+	    <browser:menuItem
+                for="zope.app.component.tests.views.IC"
+	        title="Test Entry"
+	        menu="test_menu"
+		action="@@test"
+		/>
+            '''
+	    ))
+	
 def test_suite():
     return unittest.TestSuite((
         unittest.makeSuite(Test),
