@@ -13,7 +13,7 @@
 ##############################################################################
 """Stubs for Zope RDB unit tests.
 
-$Id: stubs.py 26551 2004-07-15 07:06:37Z srichter $
+$Id: stubs.py 37758 2005-08-06 12:42:55Z hdima $
 """
 
 class ConnectionStub(object):
@@ -31,17 +31,27 @@ class ConnectionStub(object):
         v = self._called.setdefault('commit',0)
         v+=1
         self._called['commit']=v
+
     def rollback(self, *ignored):
         v = self._called.setdefault('rollback',0)
         v+=1
         self._called['rollback']=v
 
 class CursorStub(object):
+
     def execute(*args, **kw):
         pass
 
 class TypeInfoStub(object):
     paramstyle = 'pyformat'
     threadsafety = 0
+    encoding = 'utf-8'
+
+    def setEncoding(self, encoding):
+        self.encoding = encoding
+
+    def getEncoding(self):
+        return self.encoding
+
     def getConverter(self, type):
         return lambda x: x
