@@ -15,7 +15,7 @@
 
 There should be a file 'ftesting.zcml' in the current directory.
 
-$Id: functional.py 29151 2005-02-15 22:00:45Z mgedmin $
+$Id: functional.py 37964 2005-08-16 15:41:52Z benji_york $
 """
 import logging
 import re
@@ -44,12 +44,13 @@ import zope.app.testing.setup
 from zope.app import zapi
 from zope.app.debug import Debugger
 from zope.app.publication.http import HTTPPublication
-from zope.app.publication.browser import BrowserPublication
+from zope.app.publication.browser import BrowserPublication, setDefaultSkin
 from zope.app.publication.xmlrpc import XMLRPCPublication
 from zope.app.publication.soap import SOAPPublication
 from zope.app.publication.interfaces import ISOAPRequestFactory
 from zope.app.publication.zopepublication import ZopePublication
 from zope.app.publication.http import HTTPPublication
+from zope.app.publication.httpfactory import chooseClasses
 from zope.publisher.interfaces.browser import IDefaultBrowserLayer
 from zope.publisher.interfaces.browser import IDefaultSkin
 from zope.publisher.interfaces.browser import IBrowserRequest
@@ -87,18 +88,12 @@ class ResponseWrapper(object):
         return getattr(self._response, attr)
 
 
-def _getDefaultSkin():
-    """Returns the current default skin as an interface."""
-    adapters = zapi.getSiteManager().adapters
-    skin = adapters.lookup((IBrowserRequest,), IDefaultSkin, '')
-    return skin or IDefaultBrowserLayer
-
 class IManagerSetup(zope.interface.Interface):
     """Utility for enabling up a functional testing manager with needed grants
 
-    XXX This is an interim solution.  It tries to break the dependence
+    TODO This is an interim solution.  It tries to break the dependence
     on a particular security policy, however, we need a much better
-    way of managing functional-testing configurations.    
+    way of managing functional-testing configurations.
     """
 
     def setUpManager():
@@ -139,9 +134,9 @@ class FunctionalTestSetup(object):
             setup = component.queryUtility(IManagerSetup)
             if setup is not None:
                 setup.setUpManager()
-            
+
             FunctionalTestSetup().connection = None
-            
+
         elif config_file and config_file != self._config_file:
             # Running different tests with different configurations is not
             # supported at the moment
@@ -228,7 +223,7 @@ class CookieHandler(object):
             if self.cookies[k].has_key('Path'):
                 self.cookies[k]['Path'] = v['Path']
 
-                 
+
 class BrowserTestCase(CookieHandler, FunctionalTestCase):
     """Functional test case for Browser requests."""
 
@@ -270,7 +265,7 @@ class BrowserTestCase(CookieHandler, FunctionalTestCase):
                                environment=environment,
                                basic=basic, form=form,
                                request=BrowserRequest)
-        interface.directlyProvides(request, _getDefaultSkin())
+        setDefaultSkin(request)
         return request
 
     def publish(self, path, basic=None, form=None, env={},
@@ -346,7 +341,7 @@ class BrowserTestCase(CookieHandler, FunctionalTestCase):
                 a = base + a
             if a.find('#') != -1:
                 a = a[:a.index('#') - 1]
-            # XXX what about queries (/path/to/foo?bar=baz&etc)?
+            # ??? what about queries (/path/to/foo?bar=baz&etc)?
             request = None
             try:
                 try:
@@ -357,14 +352,20 @@ class BrowserTestCase(CookieHandler, FunctionalTestCase):
                     object = publication.getApplication(request)
                     object = request.traverse(object)
                     publication.afterTraversal(request, object)
-                except (KeyError, NameError, AttributeError, Unauthorized, Forbidden):
-                    e = traceback.format_exception_only(*sys.exc_info()[:2])[-1]
+                except (KeyError, NameError, AttributeError, Unauthorized,
+                        Forbidden):
+                    e = traceback.format_exception_only(
+                        *sys.exc_info()[:2])[-1]
                     errors.append((a, e.strip()))
             finally:
                 publication.endRequest(request, object)
                 self.setSite(old_site)
-                # Bad Things(TM) related to garbage collection and special
-                # __del__ methods happen if request.close() is not called here
+
+                # Make sure we don't have pending changes
+                abort()
+                
+                # The request should always be closed to free resources
+                # held by the request
                 if request:
                     request.close()
         if errors:
@@ -436,7 +437,7 @@ class HTTPHeaderOutput:
         self.headersl = []
         self.protocol = protocol
         self.omit = omit
-    
+
     def setResponseStatus(self, status, reason):
         self.status, self.reason = status, reason
 
@@ -537,7 +538,7 @@ def sample_test_suite():
 class HTTPCaller(CookieHandler):
     """Execute an HTTP request string via the publisher"""
 
-    def __call__(self, request_string, handle_errors=True):
+    def __call__(self, request_string, handle_errors=True, form=None):
         # Commit work done by previous python code.
         commit()
 
@@ -575,41 +576,26 @@ class HTTPCaller(CookieHandler):
 
         old_site = getSite()
         setSite(None)
+
+        request_cls, publication_cls = self.chooseRequestClass(method, path,
+                                                               environment)
         app = FunctionalTestSetup().getApplication()
-        header_output = HTTPHeaderOutput(
-            protocol, ('x-content-type-warning', 'x-powered-by'))
-
-        content_type = environment.get('CONTENT_TYPE', '')
-        is_xml = content_type.startswith('text/xml')
-
-        if method in ('GET', 'POST', 'HEAD'):
-            if (method == 'POST' and environment.get('HTTP_SOAPACTION', None)
-                and is_xml):
-                factory = zapi.queryUtility(ISOAPRequestFactory)
-                if factory is not None:
-                    request_cls = factory(StringIO(), StringIO(), {}).__class__
-                    publication_cls = SOAPPublication
-                else:
-                    request_cls = BrowserRequest
-                    publication_cls = BrowserPublication
-            elif (method == 'POST' and is_xml):
-                request_cls = XMLRPCRequest
-                publication_cls = XMLRPCPublication
-            else:
-                request_cls = BrowserRequest
-                publication_cls = BrowserPublication
-            
-        else:
-            request_cls = HTTPRequest
-            publication_cls = HTTPPublication
 
         request = app._request(
             path, instream, outstream,
             environment=environment,
             request=request_cls, publication=publication_cls)
-        if request_cls is BrowserRequest:
-            # Only browser requests have skins
-            interface.directlyProvides(request, _getDefaultSkin())
+        if IBrowserRequest.providedBy(request):
+            # only browser requests have skins
+            setDefaultSkin(request)
+
+        if form is not None:
+            if request.form:
+                raise ValueError("only one set of form values can be provided")
+            request.form = form
+
+        header_output = HTTPHeaderOutput(
+            protocol, ('x-content-type-warning', 'x-powered-by'))
         request.response.setHeaderOutput(header_output)
         response = DocResponseWrapper(
             request.response, outstream, path, header_output)
@@ -623,6 +609,12 @@ class HTTPCaller(CookieHandler):
 
         return response
 
+    def chooseRequestClass(self, method, path, environment):
+        """Choose and return a request class and a publication class"""
+        # note that `path` is unused by the default implementation (BBB)
+        return chooseClasses(method, environment)
+ 
+
 def FunctionalDocFileSuite(*paths, **kw):
     globs = kw.setdefault('globs', {})
     globs['http'] = HTTPCaller()
@@ -634,7 +626,7 @@ def FunctionalDocFileSuite(*paths, **kw):
     kwsetUp = kw.get('setUp')
     def setUp(test):
         FunctionalTestSetup().setUp()
-        
+
         if kwsetUp is not None:
             kwsetUp(test)
     kw['setUp'] = setUp
@@ -647,7 +639,10 @@ def FunctionalDocFileSuite(*paths, **kw):
     kw['tearDown'] = tearDown
 
     if 'optionflags' not in kw:
-        kw['optionflags'] = (doctest.ELLIPSIS
+        old = doctest.set_unittest_reportflags(0)
+        doctest.set_unittest_reportflags(old)
+        kw['optionflags'] = (old
+                             | doctest.ELLIPSIS
                              | doctest.REPORT_NDIFF
                              | doctest.NORMALIZE_WHITESPACE)
 
