@@ -13,14 +13,18 @@
 ##############################################################################
 """WSGI tests
 
-$Id: tests.py 28454 2004-11-15 10:59:19Z eckart $
+$Id: tests.py 29956 2005-04-12 20:34:35Z srichter $
 """
 import unittest
-from zope.testing.doctestunit import DocTestSuite
+from zope.testing import doctest
+from zope.app.testing import placelesssetup
 
 def test_suite():
     return unittest.TestSuite((
-        DocTestSuite('zope.app.wsgi'),
+        doctest.DocFileSuite('README.txt',
+                             setUp=placelesssetup.setUp,
+                             tearDown=placelesssetup.tearDown,
+                             optionflags=doctest.NORMALIZE_WHITESPACE),
         ))
 
 if __name__ == '__main__':
