@@ -15,7 +15,7 @@
 
 This implementation is based on a notion of "surrogate" interfaces.
 
-$Id: adapter.py 29315 2005-02-26 16:54:05Z srichter $
+$Id: adapter.py 37594 2005-07-30 13:32:05Z jim $
 """
 
 # Implementation notes
@@ -123,6 +123,7 @@ class Surrogate(object):
 
     def __init__(self, spec, registry):
         self.spec = spec.weakref()
+        self.registry = registry
         spec.subscribe(self)
         self.adapters = {}
         self.dependents = weakref.WeakKeyDictionary()
@@ -138,6 +139,18 @@ class Surrogate(object):
             del self.selfImplied
             del self.multImplied
             del self.get
+
+        bases = [self.registry.get(base) for base in self.spec().__bases__]
+        if bases != self.__bases__:
+            # Our bases changed. unsubscribe from the old ones
+            # and subscribe to the new ones
+            for base in self.__bases__:
+                base.unsubscribe(self)
+
+            self.__bases__ = bases
+            for base in bases:
+                base.subscribe(self)
+
         for dependent in self.dependents.keys():
             dependent.dirty()
 
@@ -406,7 +419,6 @@ class AdapterLookup(object):
             adapter = factory(object)
             if adapter is not None:
                 return adapter
-
         return default
 
     def queryAdapter(self, object, interface, name='', default=None):
@@ -414,7 +426,6 @@ class AdapterLookup(object):
         # We usually end up calling adapter_hook
         return self.adapter_hook(interface, object, name, default)
 
-
     def subscriptions(self, required, provided):
         if provided is None:
             provided = Null
@@ -473,7 +484,10 @@ class AdapterLookup(object):
 
     def subscribers(self, objects, interface):
         subscriptions = self.subscriptions(map(providedBy, objects), interface)
-        return [subscription(*objects) for subscription in subscriptions]
+        subscribers = [subscription(*objects)
+                       for subscription in subscriptions]
+        # Filter None values
+        return [x for x in subscribers if x is not None]
 
     def get(self, declaration):
         if declaration is None:
