@@ -23,13 +23,14 @@ messages about granted attribute access.
 Note that the ZOPE_WATCH_CHECKERS mechanism will eventually be
 replaced with a more general security auditing mechanism.
 
-$Id: checker.py 29253 2005-02-23 00:53:50Z mgedmin $
+$Id: checker.py 37672 2005-08-02 23:10:08Z Zen $
 """
 import os
 import sys
 import sets
 import types
 import datetime
+import pytz
 import weakref
 
 from zope.exceptions import DuplicationError
@@ -193,6 +194,8 @@ class Checker(object):
 
     def proxy(self, value):
         'See IChecker'
+        if type(value) is Proxy:
+            return value
         checker = getattr(value, '__Security_checker__', None)
         if checker is None:
             checker = selectChecker(value)
@@ -571,6 +574,8 @@ BasicTypes = {
     datetime.datetime: NoProxy,
     datetime.date: NoProxy,
     datetime.time: NoProxy,
+    datetime.tzinfo: NoProxy,
+    type(pytz.UTC): NoProxy,
 }
 
 # Available for tests. Located here so it can be kept in sync with BasicTypes.
@@ -654,6 +659,14 @@ _default_checkers = {
     zope.interface.declarations.Declaration: _Declaration_checker,
 }
 
+# If we are running with Python 2.4+, setup security on the builtin
+# set and frozenset types.
+try:
+    _default_checkers[set] = _setChecker
+    _default_checkers[frozenset] = _setChecker
+except NameError:
+    pass
+
 def _clear():
     _checkers.clear()
     _checkers.update(_default_checkers)
