@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.3
+#!/usr/bin/env python
 """
 A script to start the schoolbell server from the source directory.
 """
@@ -26,4 +26,4 @@ sys.path.insert(0, os.path.join(basedir, 'src'))
 sys.path.insert(0, os.path.join(basedir, 'Zope3', 'src'))
 
 import schoolbell.app.main
-schoolbell.app.main.main()
+schoolbell.app.main.StandaloneServer().main()
