@@ -19,10 +19,13 @@
 """
 SchoolBell application views.
 
-$Id: app.py 3237 2005-03-08 17:05:16Z jinty $
+$Id: app.py 5021 2005-09-20 09:24:55Z mg $
 """
 
+import itertools
+
 from zope.interface import Interface, implements
+from zope.component import adapts
 from zope.schema import Password, TextLine, Bytes, Bool, getFieldNamesInOrder
 from zope.schema import Choice
 from zope.schema.interfaces import ValidationError
@@ -46,15 +49,44 @@ from zope.security.management import getSecurityPolicy
 
 from schoolbell import SchoolBellMessageID as _
 from schoolbell.app.interfaces import IGroupMember, IPerson, IResource
-from schoolbell.app.interfaces import IPersonPreferences
+from schoolbell.app.interfaces import IPersonPreferences, IPersonDetails
 from schoolbell.app.interfaces import IPersonContainer, IPersonContained
 from schoolbell.app.interfaces import IGroupContainer, IGroupContained
 from schoolbell.app.interfaces import IResourceContainer, IResourceContained
 from schoolbell.app.interfaces import ISchoolBellApplication
+from schoolbell.app.interfaces import IApplicationPreferences
+from schoolbell.app.interfaces import vocabulary
 from schoolbell.app.app import Person
 from schoolbell.app.app import getSchoolBellApplication
+from schoolbell.app.browser.cal import CalendarOwnerTraverser
+
+from schoolbell.batching import Batch
+from schoolbell.batching.browser import MultiBatchViewMixin
+
+from pytz import common_timezones
+
+
+class SchoolBellApplicationTraverser(CalendarOwnerTraverser):
+    """Traverser for a SchoolBellApplication."""
+
+    adapts(ISchoolBellApplication)
+
+    def publishTraverse(self, request, name):
+        if name in ('persons', 'resources', 'groups'):
+            return self.context[name]
+
+        return CalendarOwnerTraverser.publishTraverse(self, request, name)
+
+
+class SchoolBellApplicationView(BrowserView):
+    """A view for the main application."""
+
+    def update(self):
+        prefs = IApplicationPreferences(getSchoolBellApplication())
+        if prefs.frontPageCalendar:
+            url = zapi.absoluteURL(self.context.calendar, self.request)
+            self.request.response.redirect(url)
 
-import pytz
 
 class ContainerView(BrowserView):
     """A base view for all containers.
@@ -68,6 +100,19 @@ class ContainerView(BrowserView):
 
     """
 
+    def update(self):
+        if 'SEARCH' in self.request and 'CLEAR_SEARCH' not in self.request:
+            searchstr = self.request['SEARCH'].lower()
+            results = [item for item in self.context.values()
+                       if searchstr in item.title.lower()]
+        else:
+            self.request.form['SEARCH'] = ''
+            results = self.context.values()
+
+        start = int(self.request.get('batch_start', 0))
+        size = int(self.request.get('batch_size', 10))
+        self.batch = Batch(results, start, size, sort_by='title')
+
 
 class PersonContainerView(ContainerView):
     """A Person Container view."""
@@ -99,11 +144,47 @@ class ResourceContainerView(ContainerView):
     add_url = "+/addSchoolBellResource.html"
 
 
+class ContainerDeleteView(BrowserView):
+    """A view for deleting items from container."""
+
+    def listIdsForDeletion(self):
+        return [key for key in self.context
+                if "delete.%s" % key in self.request]
+
+    def _listItemsForDeletion(self):
+        return [self.context[key] for key in self.listIdsForDeletion()]
+
+    itemsToDelete = property(_listItemsForDeletion)
+
+    def update(self):
+        if 'UPDATE_SUBMIT' in self.request:
+            for key in self.listIdsForDeletion():
+                del self.context[key]
+            self.request.response.redirect(self.nextURL())
+        elif 'CANCEL' in self.request:
+            self.request.response.redirect(self.nextURL())
+
+    def nextURL(self):
+        return zapi.absoluteURL(self.context, self.request)
+
+
+class PersonContainerDeleteView(ContainerDeleteView):
+    """A view for deleting users from PersonContainer."""
+
+    def isDeletingHimself(self):
+        person = IPerson(self.request.principal, None)
+        return person in self.itemsToDelete
+
+
 class PersonView(BrowserView):
     """A Person info view."""
 
     __used_for__ = IPersonContained
 
+    def __init__(self, context, request):
+        BrowserView.__init__(self, context, request)
+        self.details = IPersonDetails(self.context)
+
 
 class PersonPhotoView(BrowserView):
     """View that returns photo of a Person."""
@@ -123,32 +204,50 @@ class GroupListView(BrowserView):
 
     __used_for__ = IGroupMember
 
-    def getAllGroups(self):
-        """Return a list of groups the current user can add to"""
+    def getCurrentGroups(self):
+        """Return a list of groups the current user is a member of."""
+        return self.context.groups
+
+    def getPotentialGroups(self):
+        """Return a list of groups the current user is not a member of."""
         groups = ISchoolBellApplication(self.context)['groups']
         return [group for group in groups.values()
-                if checkPermission('schoolbell.manageMembership', group)]
+                if checkPermission('schoolbell.manageMembership', group)
+                and group not in self.context.groups]
 
     def update(self):
         context_url = zapi.absoluteURL(self.context, self.request)
-        if 'UPDATE_SUBMIT' in self.request:
+        if 'ADD_GROUPS' in self.request:
             context_groups = removeSecurityProxy(self.context.groups)
-            for group in self.getAllGroups():
-                want = bool('group.' + group.__name__ in self.request)
-                have = bool(group in context_groups)
-                # add() and remove() could throw an exception, but at the
-                # moment the constraints are never violated, so we ignore
-                # the problem.
-                if want != have:
+            for group in self.getPotentialGroups():
+                # add() could throw an exception, but at the moment the
+                # constraints are never violated, so we ignore the problem.
+                if 'add_group.' + group.__name__ in self.request:
                     group = removeSecurityProxy(group)
-                    if want:
-                        context_groups.add(group)
-                    else:
-                        context_groups.remove(group)
-            self.request.response.redirect(context_url)
+                    context_groups.add(group)
+        elif 'REMOVE_GROUPS' in self.request:
+            context_groups = removeSecurityProxy(self.context.groups)
+            for group in self.getCurrentGroups():
+                # add() could throw an exception, but at the moment the
+                # constraints are never violated, so we ignore the problem.
+                if 'remove_group.' + group.__name__ in self.request:
+                    group = removeSecurityProxy(group)
+                    context_groups.remove(group)
         elif 'CANCEL' in self.request:
             self.request.response.redirect(context_url)
 
+        if 'SEARCH' in self.request and 'CLEAR_SEARCH' not in self.request:
+            searchstr = self.request['SEARCH'].lower()
+            results = [item for item in self.getPotentialGroups()
+                       if searchstr in item.title.lower()]
+        else:
+            self.request.form['SEARCH'] = ''
+            results = self.getPotentialGroups()
+
+        start = int(self.request.get('batch_start', 0))
+        size = int(self.request.get('batch_size', 10))
+        self.batch = Batch(results, start, size, sort_by='title')
+
 
 class GroupView(BrowserView):
     """A Group info view."""
@@ -162,7 +261,7 @@ class GroupView(BrowserView):
         return filter(IResource.providedBy, self.context.members)
 
 
-class MemberViewBase(BrowserView):
+class MemberViewPersons(BrowserView):
     """A base view class for adding / removing members from a group.
 
     Subclasses must override container_name.
@@ -170,45 +269,54 @@ class MemberViewBase(BrowserView):
 
     __used_for__ = IGroupContained
 
-    container_name = None
+    container_name = 'persons'
+
+    def getMembers(self):
+        """Return a list of current group memebers."""
+        return filter(IPerson.providedBy, self.context.members)
 
     def getPotentialMembers(self):
         """Return a list of all possible members."""
         container = ISchoolBellApplication(self.context)[self.container_name]
-        return container.values()
+        return [m for m in container.values() if m not in self.context.members]
+
+    def searchPotentialMembers(self, s):
+        potentials = self.getPotentialMembers()
+        return [m for m in potentials if s.lower() in m.title.lower()]
+
+    def updateBatch(self, lst):
+        start = int(self.request.get('batch_start', 0))
+        size = int(self.request.get('batch_size', 10))
+        self.batch = Batch(lst, start, size, sort_by='title')
 
     def update(self):
-        # This method is rather similar to GroupListView.update().
         context_url = zapi.absoluteURL(self.context, self.request)
-        if 'UPDATE_SUBMIT' in self.request:
+        if 'DONE' in self.request:
+            self.request.response.redirect(context_url)
+        elif 'ADD_MEMBERS' in self.request:
             context_members = removeSecurityProxy(self.context.members)
             for member in self.getPotentialMembers():
-                want = bool('member.' + member.__name__ in self.request)
-                have = bool(member in context_members)
-                # add() and remove() could throw an exception, but at the
-                # moment the constraints are never violated, so we ignore
-                # the problem.
-                if want != have:
+                # add() could throw an exception, but at the moment the
+                # constraints are never violated, so we ignore the problem.
+                if 'ADD_MEMBER.' + member.__name__ in self.request:
                     member = removeSecurityProxy(member)
-                    if want:
-                        context_members.add(member)
-                    else:
-                        context_members.remove(member)
-            self.request.response.redirect(context_url)
-        elif 'CANCEL' in self.request:
-            self.request.response.redirect(context_url)
+                    context_members.add(member)
+        elif 'REMOVE_MEMBERS' in self.request:
+            context_members = removeSecurityProxy(self.context.members)
+            for member in self.getMembers():
+                # remove() could throw an exception, but at the moment the
+                # constraints are never violated, so we ignore the problem.
+                if 'REMOVE_MEMBER.' + member.__name__ in self.request:
+                    member = removeSecurityProxy(member)
+                    context_members.remove(member)
 
+        results = self.getPotentialMembers()
+        if 'SEARCH' in self.request and 'CLEAR_SEARCH' not in self.request:
+            results = self.searchPotentialMembers(self.request.get('SEARCH'))
+        else:
+            self.request.form['SEARCH'] = ''
 
-class MemberViewPersons(MemberViewBase):
-    """A view for adding / removing group members that are persons."""
-
-    container_name = 'persons'
-
-
-class MemberViewResources(MemberViewBase):
-    """A view for adding / removing group members that are resources."""
-
-    container_name = 'resources'
+        self.updateBatch(results)
 
 
 class ResourceView(BrowserView):
@@ -220,22 +328,28 @@ class ResourceView(BrowserView):
 class IPersonEditForm(Interface):
     """Schema for a person's edit form."""
 
-    title = TextLine(title=u"Full name",
-                     description=u"Name that should be displayed")
+    title = TextLine(
+        title=_("Full name"),
+        description=_("Name that should be displayed"))
 
-    photo = Bytes(title=u"New photo",
-                  required=False,
-                  description=u"""Photo (in JPEG format)""")
+    photo = Bytes(
+        title=_("New photo"),
+        required=False,
+        description=_(
+            """A small picture (about 48x48 pixels in JPEG format)"""))
 
-    clear_photo = Bool(title=u'Clear photo',
-                       required=False,
-                       description=u"""Check this to clear the photo""")
+    clear_photo = Bool(
+        title=_("Clear photo"),
+        required=False,
+        description=_("""Check this to clear the photo"""))
 
-    new_password = Password(title=u"New password",
-                            required=False)
+    new_password = Password(
+        title=_("New password"),
+        required=False)
 
-    verify_password = Password(title=u"Verify password",
-                               required=False)
+    verify_password = Password(
+        title=_("Verify password"),
+        required=False)
 
 
 class PersonEditView(BrowserView):
@@ -283,21 +397,6 @@ class PersonEditView(BrowserView):
             self.request.response.redirect(url)
 
 
-class IPersonPreferencesForm(Interface):
-
-    timeformat = Choice(title=u"Time Format",
-                    description=u"Time Format",
-                    values=("HH:MM", "H:MM am/pm"))
-
-    dateformat = Choice(title=u"Date Format",
-                    description=u"Date Format",
-                    values=("MM/DD/YY", "YYYY-DD-MM", "Day Month, Year"))
-
-    weekstart = Choice(title=u"Week starts on:",
-                    description=u"Start display of weeks on Sunday or Monday",
-                    values=("Sunday", "Monday"))
-
-
 class PersonPreferencesView(BrowserView):
     """View used for editing person preferences."""
 
@@ -307,17 +406,59 @@ class PersonPreferencesView(BrowserView):
     error = None
     message = None
 
+    schema = IPersonPreferences
+
     def __init__(self, context, request):
         BrowserView.__init__(self, context, request)
 
-        prefs = IPersonPreferences(self.context)
-        initial = {'timeformat': prefs.timeformat,
-                   'dateformat': prefs.dateformat,
-                   'weekstart': prefs.weekstart}
+        prefs = self.schema(self.context)
+        initial = {}
+        for field in self.schema:
+            initial[field] = getattr(prefs, field)
 
-        setUpWidgets(self, IPersonPreferencesForm, IInputWidget,
+        setUpWidgets(self, self.schema, IInputWidget, initial=initial)
+
+    def update(self):
+        if 'CANCEL' in self.request:
+            url = zapi.absoluteURL(self.context, self.request)
+            self.request.response.redirect(url)
+        elif 'UPDATE_SUBMIT' in self.request:
+            try:
+                data = getWidgetsData(self, self.schema)
+            except WidgetsError:
+                return # Errors will be displayed next to widgets
+
+            prefs = self.schema(self.context)
+            for field in self.schema:
+                if field in data: # skip non-fields
+                    setattr(prefs, field, data[field])
+
+
+class PersonDetailsView(BrowserView):
+    """View used for editing person preferences."""
+
+    __used_for__ = IPersonDetails
+
+    error = None
+    message = None
+
+    def __init__(self, context, request):
+        BrowserView.__init__(self, context, request)
+
+        details = IPersonDetails(self.context)
+        initial = {'nickname': details.nickname,
+                   'primary_email': details.primary_email,
+                   'secondary_email': details.secondary_email,
+                   'primary_phone': details.primary_phone,
+                   'secondary_phone': details.secondary_phone,
+                   'mailing_address': details.mailing_address,
+                   'home_page': details.home_page}
+
+        setUpWidgets(self, IPersonDetails, IInputWidget,
                      initial=initial)
 
+
+
     def update(self):
         if 'CANCEL' in self.request:
             url = zapi.absoluteURL(self.context, self.request)
@@ -325,34 +466,43 @@ class PersonPreferencesView(BrowserView):
 
         if 'UPDATE_SUBMIT' in self.request:
             try:
-                data = getWidgetsData(self, IPersonPreferencesForm)
+                data = getWidgetsData(self, IPersonDetails)
             except WidgetsError:
                 return # Errors will be displayed next to widgets
 
-            prefs = IPersonPreferences(self.context)
-            prefs.timeformat = data['timeformat']
-            prefs.dateformat = data['dateformat']
-            prefs.weekstart = data['weekstart']
+            details = IPersonDetails(self.context)
+            details.nickname = data['nickname']
+            details.primary_email = data['primary_email']
+            details.secondary_email = data['secondary_email']
+            details.primary_phone = data['primary_phone']
+            details.secondary_phone = data['secondary_phone']
+            details.mailing_address = data['mailing_address']
+            details.home_page = data['home_page']
 
 
 class IPersonAddForm(Interface):
     """Schema for person adding form."""
 
-    title = TextLine(title=u"Full name",
-        description=u"Name that should be displayed")
+    title = TextLine(
+        title=_("Full name"),
+        description=_("Name that should be displayed"))
 
-    username = TextLine(title=u"Username",
-        description=u"Username")
+    username = TextLine(
+        title=_("Username"),
+        description=_("Username"))
 
-    password = Password(title=u"Password",
+    password = Password(
+        title=_("Password"),
         required=False)
 
-    verify_password = Password(title=u"Verify password",
+    verify_password = Password(
+        title=_("Verify password"),
         required=False)
 
-    photo = Bytes(title=u"Photo",
+    photo = Bytes(
+        title=_("Photo"),
         required=False,
-        description=u"""Photo (in JPEG format)""")
+        description=_("""Photo (in JPEG format)"""))
 
 
 class PersonAddView(AddView):
@@ -374,12 +524,11 @@ class PersonAddView(AddView):
     def createAndAdd(self, data):
         """Create a Person from form data and add it to the container."""
         if data['password'] != data['verify_password']:
-            self.error = u'Passwords do not match!'
-            raise WidgetsError([ValidationError('Passwords do not match!')])
+            self.error = _("Passwords do not match!")
+            raise WidgetsError([ValidationError(self.error)])
         elif data['username'] in self.context:
-            self.error = u'This username is already used!'
-            raise WidgetsError(
-                [ValidationError('This username is already used!')])
+            self.error = _('This username is already used!')
+            raise WidgetsError([ValidationError(self.error)])
         return AddView.createAndAdd(self, data)
 
     def getAllGroups(self):
@@ -505,8 +654,8 @@ class LogoutView(BrowserView):
         self.request.response.redirect(url)
 
 
-class ACLView(BrowserView):
-    """A view for editing SchoolBell-relevant local grants"""
+class ACLViewBase(object):
+    """A base view for both browser and restive access control views."""
 
     permissions = [
         ('schoolbell.view', _('View')),
@@ -518,45 +667,83 @@ class ACLView(BrowserView):
         ('schoolbell.controlAccess', _('Control access')),
         ('schoolbell.manageMembership', _('Manage membership')),
         ]
+    permission_ids = [permission for permission, title in permissions]
+    del permission, title # list comprehensions clutter local scope
 
-    def persons(self):
+    def getPersons(self):
         app = getSchoolBellApplication()
         map = IPrincipalPermissionManager(self.context)
         auth = zapi.getUtility(IAuthentication)
         result = []
         for person in app['persons'].values():
             pid = auth.person_prefix + person.__name__
-            result.append({'title': person.title, 'id': pid,
-                           'perms': self.permsForPrincipal(pid)})
+            result.append({'title': person.title, 'id': pid})
         return result
-    persons = property(persons)
+    persons = property(getPersons)
 
     def permsForPrincipal(self, principalid):
-        """Return a list of permissions allowed for principal"""
-        return [perm
-                for perm, title in self.permissions
-                if hasPermission(perm, self.context, principalid)]
+        """Return a list of permissions allowed for principal."""
+        permission_bits = hasPermissions(self.permission_ids, self.context,
+                                         principalid)
+        return [perm for perm, has in zip(self.permission_ids, permission_bits)
+                if has]
 
-    def groups(self):
+    def getGroups(self):
         app = getSchoolBellApplication()
         auth = zapi.getUtility(IAuthentication)
         map = IPrincipalPermissionManager(self.context)
         result = []
         all = zapi.queryUtility(IAuthenticatedGroup)
         if all is not None:
-            result.append({'title': 'Authenticated users', 'id': all.id,
-                           'perms': self.permsForPrincipal(all.id)})
+            result.append({'title': _('Authenticated users'),
+                           'id': all.id})
         unauth = zapi.queryUtility(IUnauthenticatedGroup)
         if unauth is not None:
-            result.append({'title': 'Unauthenticated users', 'id': unauth.id,
-                           'perms': self.permsForPrincipal(unauth.id)})
+            result.append({'title': _('Unauthenticated users'),
+                           'id': unauth.id})
         for group in app['groups'].values():
             pid = auth.group_prefix + group.__name__
             result.append({'title': group.title,
-                           'id': pid,
-                           'perms': self.permsForPrincipal(pid)})
+                           'id': pid})
         return result
-    groups = property(groups)
+    groups = property(getGroups)
+
+    def applyPermissionChanges(self, principalid, permissions):
+        """Apply new permission settings for a single principal.
+
+        Make it so `principal` has exactly the permissions specified
+        in `permissions` (and not more) on self.context.
+
+        If the requested permission grant on self.context matches the
+        one on self.context.__parent__, this method removes any
+        specific local grants from self.context.  Otherwise, it adds a
+        local grant that either grants or denies the permission
+        directly on self.context.
+        """
+        parent = self.context.__parent__
+        manager = IPrincipalPermissionManager(self.context)
+        # ACL views are protected by schoolbell.controlAccess, so
+        # removeSecurityProxy does not lead to privilege escalation
+        # problems.
+        manager = removeSecurityProxy(manager)
+        permission_bits = hasPermissions(self.permission_ids, parent,
+                                         principalid)
+        for permission, in_parent in zip(self.permission_ids, permission_bits):
+            requested = permission in permissions
+            if requested and not in_parent:
+                manager.grantPermissionToPrincipal(permission, principalid)
+            elif not requested and in_parent:
+                manager.denyPermissionToPrincipal(permission, principalid)
+            else:
+                manager.unsetPermissionForPrincipal(permission, principalid)
+
+
+class ACLView(BrowserView, ACLViewBase, MultiBatchViewMixin):
+    """A view for editing SchoolBell-relevant local grants"""
+
+    def __init__(self, context, request):
+        BrowserView.__init__(self, context, request)
+        MultiBatchViewMixin.__init__(self, ['groups', 'persons'])
 
     def update(self):
         if 'UPDATE_SUBMIT' in self.request or 'CANCEL' in self.request:
@@ -564,50 +751,79 @@ class ACLView(BrowserView):
             self.request.response.redirect(url)
 
         if 'UPDATE_SUBMIT' in self.request:
-            map = IPrincipalPermissionManager(self.context)
-            # this view is protected by schooltool.controlAccess
-            map = removeSecurityProxy(map)
-            auth = zapi.getUtility(IAuthentication)
-
-            def permChecked(perm, principalid):
-                """Returns if a checkbox for (perm, principalid) is checked"""
-                if principalid in self.request:
-                    return (perm in self.request[principalid] or
-                            perm == self.request[principalid])
-                return False
-
             for info in self.persons + self.groups:
                 principalid = info['id']
                 if 'marker-' + principalid not in self.request:
                     continue # skip this principal
-                for perm, permtitle in self.permissions:
-                    parent = self.context.__parent__
-                    checked_in_request = permChecked(perm, principalid)
-                    grant_in_parent = hasPermission(perm, parent, principalid)
-                    if checked_in_request and not grant_in_parent:
-                        map.grantPermissionToPrincipal(perm, principalid)
-                    elif not checked_in_request and grant_in_parent:
-                        map.denyPermissionToPrincipal(perm, principalid)
-                    else:
-                        map.unsetPermissionForPrincipal(perm, principalid)
+                permissions = self.request.get(principalid, [])
+                if isinstance(permissions, basestring):
+                    permissions = [permissions]
+                self.applyPermissionChanges(principalid, permissions)
+
+        MultiBatchViewMixin.update(self)
+
+        self.updateBatch('persons', self.persons)
+        self.updateBatch('groups', self.groups)
+
+        for infodict in itertools.chain(self.batches['persons'],
+                                        self.batches['groups']):
+            infodict['perms'] = self.permsForPrincipal(infodict['id'])
 
     def __call__(self):
         self.update()
         return self.index()
 
 
+class ApplicationPreferencesView(BrowserView):
+    """View used for editing application preferences."""
+
+    __used_for__ = IApplicationPreferences
+
+    error = None
+    message = None
+
+    schema = IApplicationPreferences
+
+    def __init__(self, context, request):
+        BrowserView.__init__(self, context, request)
+
+        app = getSchoolBellApplication()
+        prefs = self.schema(app)
+        initial = {}
+        for field in self.schema:
+            initial[field] = getattr(prefs, field)
+
+        setUpWidgets(self, self.schema, IInputWidget, initial=initial)
+
+    def update(self):
+        if 'CANCEL' in self.request:
+            url = zapi.absoluteURL(self.context, self.request)
+            self.request.response.redirect(url)
+        elif 'UPDATE_SUBMIT' in self.request:
+            try:
+                data = getWidgetsData(self, self.schema)
+            except WidgetsError:
+                return # Errors will be displayed next to widgets
+
+            app = getSchoolBellApplication()
+            prefs = self.schema(app)
+            for field in self.schema:
+                if field in data: # skip non-fields
+                    setattr(prefs, field, data[field])
+
+
 class ProbeParticipation:
-    """A stub participation for use in hasPermission"""
+    """A stub participation for use in hasPermissions."""
     implements(IParticipation)
     def __init__(self, principal):
         self.principal = principal
         self.interaction = None
 
 
-def hasPermission(permission, object, principalid):
-    """Returns if the principal has access according to the security policy"""
-
+def hasPermissions(permissions, object, principalid):
+    """Test if the principal has access according to the security policy."""
     principal = zapi.getUtility(IAuthentication).getPrincipal(principalid)
     participation = ProbeParticipation(principal)
     interaction = getSecurityPolicy()(participation)
-    return interaction.checkPermission(permission, object)
+    return [interaction.checkPermission(permission, object)
+            for permission in permissions]
