@@ -19,7 +19,7 @@
 """
 Tests for schoolbell views.
 
-$Id: test_app.py 3237 2005-03-08 17:05:16Z jinty $
+$Id: test_app.py 5021 2005-09-20 09:24:55Z mg $
 """
 
 import unittest
@@ -37,6 +37,242 @@ from schoolbell.app.browser.tests.setup import setUp, tearDown
 from schoolbell.app.browser.tests.setup import setUpSessions
 
 
+def doctest_SchoolBellApplicationView():
+    r"""Test for SchoolBellApplicationView
+
+    Some setup
+
+        >>> from schoolbell.app.app import SchoolBellApplication
+        >>> from zope.app.component.site import LocalSiteManager
+        >>> from zope.app.component.hooks import setSite
+        >>> from schoolbell.app.app import getApplicationPreferences
+        >>> from zope.app.annotation.interfaces import IAnnotations
+        >>> from schoolbell.app.interfaces import IApplicationPreferences
+        >>> from schoolbell.app.interfaces import ISchoolBellApplication
+        >>> from schoolbell.app.app import SchoolBellApplication
+
+        >>> app = SchoolBellApplication()
+
+        >>> app.setSiteManager(LocalSiteManager(app))
+        >>> setup.setUpAnnotations()
+        >>> setSite(app)
+
+        >>> ztapi.provideAdapter(ISchoolBellApplication,
+        ...                      IApplicationPreferences,
+        ...                      getApplicationPreferences)
+
+        >>> directlyProvides(app, IContainmentRoot)
+
+    Now lets create a view
+
+        >>> from schoolbell.app.browser.app import SchoolBellApplicationView
+        >>> request = TestRequest()
+        >>> view = SchoolBellApplicationView(app, request)
+        >>> view.update()
+
+        >>> request.response.getStatus()
+        302
+        >>> request.response.getHeaders()['Location']
+        'http://127.0.0.1/calendar'
+
+    If we change a the front page preference, we should not be redirected
+
+        >>> IApplicationPreferences(app).frontPageCalendar = False
+        >>> request = TestRequest()
+        >>> view = SchoolBellApplicationView(app, request)
+        >>> view.update()
+
+        >>> request.response.getStatus()
+        599
+
+    """
+
+
+def doctest_ContainerView():
+    r"""Test for ContainerView
+
+    Let's create some persons to toy with in a person container:
+
+        >>> from schoolbell.app.browser.app import ContainerView
+        >>> from schoolbell.app.app import Person, PersonContainer
+        >>> from schoolbell.app.interfaces import IPerson
+        >>> setup.setUpAnnotations()
+
+        >>> personContainer = PersonContainer()
+        >>> directlyProvides(personContainer, IContainmentRoot)
+
+        >>> personContainer['pete'] = Person('pete', 'Pete Parrot')
+        >>> personContainer['john'] = Person('john', 'Long John')
+        >>> personContainer['frog'] = Person('frog', 'Frog Man')
+        >>> personContainer['toad'] = Person('toad', 'Taodsworth')
+        >>> request = TestRequest()
+        >>> view = ContainerView(personContainer, request)
+
+    After calling update, we should have a batch setup with everyone in it:
+
+        >>> view.update()
+        >>> [p.title for p in view.batch]
+        ['Frog Man', 'Long John', 'Pete Parrot', 'Taodsworth']
+
+
+    We can alter the batch size and starting point through the request
+
+        >>> request.form = {'batch_start': '2',
+        ...                 'batch_size': '2'}
+        >>> view.update()
+        >>> [p.title for p in view.batch]
+        ['Pete Parrot', 'Taodsworth']
+
+    We can search through the request:
+
+        >>> request.form = {'SEARCH': 'frog'}
+        >>> view.update()
+        >>> [p.title for p in view.batch]
+        ['Frog Man']
+
+    And we can clear the search (which ignores any search value):
+
+        >>> request.form = {'SEARCH': 'frog',
+        ...                 'CLEAR_SEARCH': 'on'}
+        >>> view.update()
+        >>> [p.title for p in view.batch]
+        ['Frog Man', 'Long John', 'Pete Parrot', 'Taodsworth']
+
+    """
+
+
+def doctest_ContainerDeleteView():
+    r"""Test for ContainerDeleteView
+
+    Let's create some persons to delete from a person container:
+
+        >>> from schoolbell.app.browser.app import ContainerDeleteView
+        >>> from schoolbell.app.app import Person, PersonContainer
+        >>> from schoolbell.app.interfaces import IPerson
+        >>> setup.setUpAnnotations()
+
+        >>> personContainer = PersonContainer()
+        >>> directlyProvides(personContainer, IContainmentRoot)
+
+        >>> personContainer['pete'] = Person('pete', 'Pete Parrot')
+        >>> personContainer['john'] = Person('john', 'Long John')
+        >>> personContainer['frog'] = Person('frog', 'Frog Man')
+        >>> personContainer['toad'] = Person('toad', 'Taodsworth')
+        >>> request = TestRequest()
+        >>> view = ContainerDeleteView(personContainer, request)
+
+    We should have the list of all the Ids of items that are going to
+    be deleted from container:
+
+        >>> view.listIdsForDeletion()
+        []
+
+    We must pass ids of selected people in the request:
+
+        >>> request.form = {'delete.pete': 'on',
+        ...                 'delete.john': 'on',
+        ...                 'UPDATE_SUBMIT': 'Delete'}
+        >>> ids = [key for key in view.listIdsForDeletion()]
+        >>> ids.sort()
+        >>> ids
+        [u'john', u'pete']
+        >>> [item.title for item in view.itemsToDelete]
+        ['Long John', 'Pete Parrot']
+
+    These two should be gone after update:
+
+        >>> view.update()
+        >>> ids = [key for key in personContainer]
+        >>> ids.sort()
+        >>> ids
+        [u'frog', u'toad']
+
+    And we should be redirected to the container view:
+
+        >>> request.response.getStatus()
+        302
+        >>> request.response.getHeaders()['Location']
+        'http://127.0.0.1'
+
+    If we press Cancel no one should get hurt though:
+
+        >>> request.form = {'delete.frog': 'on',
+        ...                 'delete.toad': 'on',
+        ...                 'CANCEL': 'Cancel'}
+
+    You see, both our firends are still in there:
+
+        >>> ids = [key for key in personContainer]
+        >>> ids.sort()
+        >>> ids
+        [u'frog', u'toad']
+
+    But we should be redirected to the container:
+
+        >>> request.response.getStatus()
+        302
+        >>> request.response.getHeaders()['Location']
+        'http://127.0.0.1'
+
+    No redirection if nothing was pressed should happen:
+
+        >>> request.form = {'delete.frog': 'on',
+        ...                 'delete.toad': 'on'}
+        >>> view.update()
+        >>> request.response.getHeaders()['Location']
+        'http://127.0.0.1'
+
+    """
+
+
+def doctest_PersonContainerDeleteView():
+    r"""Test for PersonContainerDeleteView
+
+    Let's create some persons to delete from a person container:
+
+        >>> from schoolbell.app.browser.app import PersonContainerDeleteView
+        >>> from schoolbell.app.app import Person, PersonContainer
+        >>> from schoolbell.app.interfaces import IPerson
+        >>> setup.setUpAnnotations()
+
+        >>> personContainer = PersonContainer()
+        >>> directlyProvides(personContainer, IContainmentRoot)
+
+        >>> personContainer['pete'] = Person('pete', 'Pete Parrot')
+        >>> personContainer['john'] = Person('john', 'Long John')
+        >>> personContainer['frog'] = Person('frog', 'Frog Man')
+        >>> personContainer['toad'] = Person('toad', 'Taodsworth')
+        >>> request = TestRequest()
+        >>> view = PersonContainerDeleteView(personContainer, request)
+
+    Our user is not trying to delete anything yet:
+
+        >>> view.isDeletingHimself()
+        False
+
+    Lets log in:
+
+        >>> from schoolbell.app.security import Principal
+        >>> principal = Principal('pete', 'Pete Parrot', personContainer['pete'])
+        >>> request.setPrincipal(principal)
+
+    Even if he is trying to delete someone who is not pete:
+
+        >>> request.form = {'delete.frog': 'on',
+        ...                 'delete.toad': 'on'}
+        >>> view.isDeletingHimself()
+        False
+
+    But if he will try deleting himself - the method should return true:
+
+        >>> request.form = {'delete.pete': 'on',
+        ...                 'delete.toad': 'on'}
+        >>> view.isDeletingHimself()
+        True
+
+    """
+
+
 def doctest_PersonView():
     r"""Test for PersonView
 
@@ -44,6 +280,11 @@ def doctest_PersonView():
 
         >>> from schoolbell.app.browser.app import PersonView
         >>> from schoolbell.app.app import Person
+        >>> from schoolbell.app.app import getPersonDetails
+        >>> from schoolbell.app.interfaces import IPersonDetails
+        >>> from schoolbell.app.interfaces import IPerson
+        >>> setup.setUpAnnotations()
+        >>> ztapi.provideAdapter(IPerson, IPersonDetails, getPersonDetails)
         >>> person = Person()
         >>> request = TestRequest()
         >>> view = PersonView(person, request)
@@ -136,17 +377,24 @@ def doctest_GroupListView():
 
         >>> view.update()
 
-    First, all groups should be listed:
+    First, all groups the person is not a member of should be listed:
 
-        >>> group_titles = [g.title for g in view.getAllGroups()]
+        >>> group_titles = [g.title for g in view.getPotentialGroups()]
         >>> group_titles.sort()
         >>> group_titles
         ['Etria', 'Others', 'PoV']
 
+    As well as all groups the person is currently a member of:
+
+        >>> group_titles = [g.title for g in view.getCurrentGroups()]
+        >>> group_titles.sort()
+        >>> group_titles
+        []
+
     Let's tell the person to join PoV:
 
         >>> request = TestRequest()
-        >>> request.form = {'group.pov': 'on', 'UPDATE_SUBMIT': 'Apply'}
+        >>> request.form = {'add_group.pov': 'on', 'ADD_GROUPS': 'Apply'}
         >>> view = GroupListView(person, request)
         >>> view.update()
 
@@ -155,17 +403,10 @@ def doctest_GroupListView():
         >>> [group.title for group in person.groups]
         ['PoV']
 
-    And we should be directed to the person info page:
-
-        >>> request.response.getStatus()
-        302
-        >>> request.response.getHeaders()['Location']
-        'http://127.0.0.1/persons/ignas'
-
     Had we decided to make the guy join Etria but then changed our mind:
 
         >>> request = TestRequest()
-        >>> request.form = {'group.pov': 'on', 'group.etria': 'on',
+        >>> request.form = {'remove_group.pov': 'on', 'add_group.etria': 'on',
         ...                 'CANCEL': 'Cancel'}
         >>> view = GroupListView(person, request)
         >>> view.update()
@@ -186,24 +427,18 @@ def doctest_GroupListView():
     to The World.
 
         >>> request = TestRequest()
-        >>> request.form = {'group.the_world': 'on', 'UPDATE_SUBMIT': 'Apply'}
+        >>> request.form = {'remove_group.pov': 'on', 'REMOVE_GROUPS': 'Apply'}
         >>> view = GroupListView(person, request)
         >>> view.update()
 
     Mission successful:
 
         >>> [group.title for group in person.groups]
-        ['Others']
-
-    Yadda yadda, redirection works:
-
-        >>> request.response.getStatus()
-        302
-        >>> request.response.getHeaders()['Location']
-        'http://127.0.0.1/persons/ignas'
+        []
 
 
         >>> endInteraction()
+
     """
 
 
@@ -245,10 +480,17 @@ def doctest_MemberListView():
         >>> [g.title for g in view.getPotentialMembers()]
         ['Albertas', 'Gintas', 'Ignas']
 
+    We can search persons not in the group
+
+        >>> [g.title for g in view.searchPotentialMembers('al')]
+        ['Albertas']
+        >>> [g.title for g in view.searchPotentialMembers('i')]
+        ['Gintas', 'Ignas']
+
     Let's make Ignas a member of PoV:
 
         >>> request = TestRequest()
-        >>> request.form = {'member.ignas': 'on', 'UPDATE_SUBMIT': 'Apply'}
+        >>> request.form = {'ADD_MEMBER.ignas': 'on', 'ADD_MEMBERS': 'Apply'}
         >>> view = MemberViewPersons(pov, request)
         >>> view.update()
 
@@ -257,17 +499,15 @@ def doctest_MemberListView():
         >>> [person.title for person in pov.members]
         ['Ignas']
 
-    And we should be directed to the group info page:
+    Search again to make sure members do not appear in the results:
 
-        >>> request.response.getStatus()
-        302
-        >>> request.response.getHeaders()['Location']
-        'http://127.0.0.1/groups/pov'
+        >>> [g.title for g in view.searchPotentialMembers('as')]
+        ['Albertas', 'Gintas']
 
     We can cancel an action if we want to:
 
         >>> request = TestRequest()
-        >>> request.form = {'member.gintas': 'on', 'CANCEL': 'Cancel'}
+        >>> request.form = {'ADD_MEMBER.gintas': 'on', 'DONE': 'Done'}
         >>> view = MemberViewPersons(pov, request)
         >>> view.update()
         >>> [person.title for person in pov.members]
@@ -277,11 +517,17 @@ def doctest_MemberListView():
         >>> request.response.getHeaders()['Location']
         'http://127.0.0.1/groups/pov'
 
-    Finally, let's remove Ignas from PoV (he went home early today)
+    Let's remove Ignas from PoV (he went home early today);
+
+        >>> request = TestRequest()
+        >>> request.form = {'REMOVE_MEMBER.ignas': 'on', 'REMOVE_MEMBERS': 'Apply'}
+        >>> view = MemberViewPersons(pov, request)
+        >>> view.update()
+
     and add Albert, who came in late and has to work after-hours:
 
         >>> request = TestRequest()
-        >>> request.form = {'member.alga': 'on', 'UPDATE_SUBMIT': 'Apply'}
+        >>> request.form = {'ADD_MEMBER.alga': 'on', 'ADD_MEMBERS': 'Apply'}
         >>> view = MemberViewPersons(pov, request)
         >>> view.update()
 
@@ -290,8 +536,12 @@ def doctest_MemberListView():
         >>> [person.title for person in pov.members]
         ['Albertas']
 
-    Yadda yadda, redirection works:
+    Click 'Done' when we are finished and we go back to the group view
 
+        >>> request = TestRequest()
+        >>> request.form = {'DONE': 'Done'}
+        >>> view = MemberViewPersons(pov, request)
+        >>> view.update()
         >>> request.response.getStatus()
         302
         >>> request.response.getHeaders()['Location']
@@ -302,6 +552,30 @@ def doctest_MemberListView():
     """
 
 
+def doctest_MemberViewPersons_updateBatch():
+    r"""Test for MemberViewPersons.updateBatch.
+
+        >>> from schoolbell.app.browser.app import MemberViewPersons
+        >>> from schoolbell.app.app import Group
+        >>> from schoolbell.app.app import Person
+        >>> pov = Group('PoV')
+        >>> gintas = Person('gintas', 'Gintas')
+        >>> ignas = Person('ignas', 'Ignas')
+        >>> alga = Person('alga', 'Albertas')
+
+        >>> request = TestRequest()
+        >>> view = MemberViewPersons(pov, request)
+
+    updateBatch takes a list of persons, and creates a Batch object
+    from that list.
+
+        >>> view.updateBatch([ignas, alga, gintas])
+        >>> [p.title for p in view.batch]
+        ['Albertas', 'Gintas', 'Ignas']
+
+    """
+
+
 def doctest_GroupView():
     r"""Test for GroupView
 
@@ -759,15 +1033,65 @@ def doctest_PersonPreferencesView():
 
     Cancel a change TODO: set view.message
 
-        >>> request.form = {'CANCEL': 'Cancel'}
+        >>> request = TestRequest(form={'CANCEL': 'Cancel'})
         >>> view = PersonPreferencesView(person, request)
 
-    XXX this should test post
+    Let's see if posting works properly:
+
+        >>> request = TestRequest(form={'UPDATE_SUBMIT': 'Update',
+        ...                             'field.timezone': 'Europe/Vilnius',
+        ...                             'field.timeformat': '%H:%M',
+        ...                             'field.dateformat': '%d %B, %Y',
+        ...                             'field.weekstart': '6'})
+        >>> view = PersonPreferencesView(person, request)
+
+        >>> view.update()
+
+        >>> prefs = getPersonPreferences(person)
+        >>> prefs.timezone, prefs.timeformat, prefs.dateformat, prefs.weekstart
+        ('Europe/Vilnius', '%H:%M', '%d %B, %Y', 6)
 
     """
 
+
+def doctest_PersonDetailsView():
+    """
+
+        >>> from schoolbell.app.browser.app import PersonDetailsView
+        >>> from schoolbell.app.app import Person
+        >>> from schoolbell.app.app import getPersonDetails
+        >>> from schoolbell.app.interfaces import IPersonDetails
+        >>> from schoolbell.app.interfaces import IPerson
+
+        >>> setup.setUpAnnotations()
+        >>> ztapi.provideAdapter(IPerson, IPersonDetails, \
+                                 getPersonDetails)
+
+        >>> person = Person()
+        >>> request = TestRequest()
+
+        >>> view = PersonDetailsView(person, request)
+
+    Cancel a change TODO: set view.message
+
+        >>> request.form = {'CANCEL': 'Cancel'}
+        >>> view = PersonDetailsView(person, request)
+
+    """
+
+
 def doctest_LoginView():
     """
+
+    Some framework setup:
+
+        >>> from schoolbell.app.interfaces import ISchoolBellApplication
+        >>> from schoolbell.app.interfaces import IApplicationPreferences
+        >>> from schoolbell.app.app import getApplicationPreferences
+        >>> ztapi.provideAdapter(ISchoolBellApplication, 
+        ...                      IApplicationPreferences,
+        ...                      getApplicationPreferences)
+
     We have to set up a security checker for person objects:
 
         >>> from schoolbell.app.app import Person
@@ -940,8 +1264,126 @@ def doctest_LogoutView():
     """
 
 
-def doctest_ACLView():
-    r"""
+def doctest_ACLViewBase_applyPermissionChanges():
+    r"""Tests for ACLViewBase.applyPermissionChanges.
+
+    Import the view:
+
+        >>> from schoolbell.app.browser.app import ACLViewBase
+
+    Set up local permission grants:
+
+        >>> from zope.app.annotation.interfaces import IAnnotatable
+        >>> from zope.app.securitypolicy.interfaces import \
+        ...                         IPrincipalPermissionManager
+        >>> from zope.app.securitypolicy.principalpermission import \
+        ...                         AnnotationPrincipalPermissionManager
+        >>> setup.setUpAnnotations()
+        >>> setup.setUpTraversal()
+        >>> ztapi.provideAdapter(IAnnotatable, IPrincipalPermissionManager,
+        ...                      AnnotationPrincipalPermissionManager)
+
+    Install the Zope security policy:
+
+        >>> from zope.security.management import setSecurityPolicy
+        >>> from zope.app.securitypolicy.zopepolicy import ZopeSecurityPolicy
+        >>> policy = setSecurityPolicy(ZopeSecurityPolicy)
+
+    Set up the application:
+
+        >>> from schoolbell.app.app import SchoolBellApplication
+        >>> app = SchoolBellApplication()
+        >>> from schoolbell.app.security import setUpLocalAuth
+        >>> setUpLocalAuth(app)
+        >>> from zope.app.component.hooks import setSite
+        >>> setSite(app)
+
+    And a person:
+
+        >>> from schoolbell.app.app import Person
+        >>> app['persons']['1'] = Person('joe', title='Joe')
+
+    Create the view base:
+
+        >>> vb = ACLViewBase()
+        >>> vb.context = app['persons']
+
+    At the moment Joe can't do anything with the person container:
+
+        >>> from schoolbell.app.browser.app import hasPermissions
+        >>> hasPermissions(['schoolbell.view',
+        ...                 'schoolbell.edit',
+        ...                 'schoolbell.create'],
+        ...                app['persons'],
+        ...                'sb.person.joe')
+        [False, False, False]
+
+    Let's give him some permissions:
+
+        >>> vb.applyPermissionChanges('sb.person.joe',
+        ...                           ['schoolbell.view',
+        ...                            'schoolbell.edit'])
+
+    Now he can view and edit properties of the container:
+
+        >>> hasPermissions(['schoolbell.view',
+        ...                 'schoolbell.edit',
+        ...                 'schoolbell.create'],
+        ...                app['persons'],
+        ...                'sb.person.joe')
+        [True, True, False]
+
+    By default Joe can do with Persons anything that he can do with
+    Person container:
+
+        >>> hasPermissions(['schoolbell.view',
+        ...                 'schoolbell.edit',
+        ...                 'schoolbell.create'],
+        ...                app['persons']['joe'],
+        ...                'sb.person.joe')
+        [True, True, False]
+
+    But we can change it by granting or denying him these privilegies
+    through the same view:
+
+        >>> vb2 = ACLViewBase()
+        >>> vb2.context = app['persons']['joe']
+
+        >>> vb2.applyPermissionChanges('sb.person.joe',
+        ...                           ['schoolbell.view',
+        ...                            'schoolbell.create'])
+
+    Our user just lost his permission to edit, though now he has the
+    permission to create:
+
+        >>> hasPermissions(['schoolbell.view',
+        ...                 'schoolbell.edit',
+        ...                 'schoolbell.create'],
+        ...                app['persons']['joe'],
+        ...                'sb.person.joe')
+        [True, False, True]
+
+    View is still inherited from person container so if we will
+    retract the view permission on the container:
+
+        >>> vb.applyPermissionChanges('sb.person.joe',
+        ...                           ['schoolbell.edit'])
+
+    Joe will not be capable of viewing his own properties anymore:
+
+        >>> hasPermissions(['schoolbell.view',
+        ...                 'schoolbell.edit',
+        ...                 'schoolbell.create'],
+        ...                app['persons']['joe'],
+        ...                'sb.person.joe')
+        [False, False, True]
+
+    """
+
+
+def doctest_ACLViewBase_permsForPrincipal():
+    r"""Test for ACLViewBase.permsForPrincipal
+
     Set up for local grants:
 
         >>> from zope.app.annotation.interfaces import IAnnotatable
@@ -954,6 +1396,86 @@ def doctest_ACLView():
         >>> ztapi.provideAdapter(IAnnotatable, IPrincipalPermissionManager,
         ...                      AnnotationPrincipalPermissionManager)
 
+    Let's set the Zope security policy:
+
+        >>> from zope.security.management import setSecurityPolicy
+        >>> from zope.app.securitypolicy.zopepolicy import ZopeSecurityPolicy
+        >>> old = setSecurityPolicy(ZopeSecurityPolicy)
+
+    Suppose we have a Schoolbell object:
+
+        >>> from schoolbell.app.app import SchoolBellApplication
+        >>> app = SchoolBellApplication()
+        >>> directlyProvides(app, IContainmentRoot)
+        >>> persons = app['persons']
+        >>> from schoolbell.app.security import setUpLocalAuth
+        >>> setUpLocalAuth(app)
+        >>> from zope.app.component.hooks import setSite
+        >>> setSite(app)
+
+    In it, we have a principal:
+
+        >>> from schoolbell.app.app import Person
+        >>> joe = app['persons']['1'] = Person('joe', title='Joe')
+
+
+    Let's create an ACLViewBase:
+
+        >>> from schoolbell.app.browser.app import ACLViewBase
+        >>> vb = ACLViewBase()
+        >>> vb.context = app
+
+    He can't do anything yet:
+
+        >>> vb.permsForPrincipal('sb.person.joe')
+        []
+
+    However, we can add a local grant:
+
+        >>> perms = IPrincipalPermissionManager(app)
+        >>> perms.grantPermissionToPrincipal('schoolbell.view', 'sb.person.joe')
+
+    And everything changes!
+
+        >>> vb.permsForPrincipal('sb.person.joe')
+        ['schoolbell.view']
+
+    The same works for subobjects:
+
+        >>> vb.context = app['persons']
+        >>> vb.permsForPrincipal('sb.person.joe')
+        ['schoolbell.view']
+        >>> vb.context = app['persons']['joe']
+        >>> vb.permsForPrincipal('sb.person.joe')
+        ['schoolbell.view']
+
+    """
+
+
+# TODO: extract separate unit tests for other ACLViewBase methods
+
+
+def doctest_ACLView():
+    r"""Tests for ACLView.
+
+    Set up for local grants:
+
+        >>> from zope.app.annotation.interfaces import IAnnotatable
+        >>> from zope.app.securitypolicy.interfaces import \
+        ...                         IPrincipalPermissionManager
+        >>> from zope.app.securitypolicy.principalpermission import \
+        ...                         AnnotationPrincipalPermissionManager
+        >>> setup.setUpAnnotations()
+        >>> setup.setUpTraversal()
+        >>> ztapi.provideAdapter(IAnnotatable, IPrincipalPermissionManager,
+        ...                      AnnotationPrincipalPermissionManager)
+        >>> from schoolbell.app.interfaces import ISchoolBellApplication
+        >>> from schoolbell.app.interfaces import IApplicationPreferences
+        >>> from schoolbell.app.app import getApplicationPreferences
+        >>> ztapi.provideAdapter(ISchoolBellApplication, 
+        ...                      IApplicationPreferences,
+        ...                      getApplicationPreferences)
+
     Let's set the security policy:
 
         >>> from zope.security.management import setSecurityPolicy
@@ -990,17 +1512,17 @@ def doctest_ACLView():
         >>> request.setPrincipal(StubPrincipal())
         >>> view = View(app, request)
 
-    The view has methods to list persons:
+    The view has methods to list persons and groups:
 
         >>> pprint(view.persons)
-        [{'perms': [], 'id': u'sb.person.albert', 'title': 'Albert'},
-         {'perms': [], 'id': u'sb.person.marius', 'title': 'Marius'}]
+        [{'id': u'sb.person.albert', 'title': 'Albert'},
+         {'id': u'sb.person.marius', 'title': 'Marius'}]
         >>> pprint(view.groups)
-        [{'perms': [], 'id': u'sb.group.3', 'title': 'office'},
-         {'perms': [], 'id': u'sb.group.4', 'title': 'mgmt'}]
+        [{'id': u'sb.group.3', 'title': 'office'},
+         {'id': u'sb.group.4', 'title': 'mgmt'}]
 
     If we have an authenticated group and an unauthenticated group, we
-    get then as well:
+    get them as well:
 
         >>> from zope.app.security.interfaces import IAuthentication
         >>> from zope.app.security.interfaces import IAuthenticatedGroup
@@ -1024,10 +1546,10 @@ def doctest_ACLView():
         >>> principalRegistry.registerGroup(authenticated)
 
         >>> pprint(view.groups)
-        [{'perms': [], 'id': 'zope.authenticated', 'title': 'Authenticated users'},
-         {'perms': [], 'id': 'zope.unauthenticated', 'title': 'Unauthenticated users'},
-         {'perms': [], 'id': u'sb.group.3', 'title': 'office'},
-         {'perms': [], 'id': u'sb.group.4', 'title': 'mgmt'}]
+        [{'id': 'zope.authenticated', 'title': u'Authenticated users'},
+         {'id': 'zope.unauthenticated', 'title': u'Unauthenticated users'},
+         {'id': u'sb.group.3', 'title': 'office'},
+         {'id': u'sb.group.4', 'title': 'mgmt'}]
 
     Also it knows a list of permissions to display:
 
@@ -1042,7 +1564,7 @@ def doctest_ACLView():
          ('schoolbell.manageMembership', u'Manage membership')]
 
     The view displays a matrix with groups and persons as rows and
-    permisssions as columns:
+    permissions as columns:
 
         >>> print view()
         <BLANKLINE>
@@ -1050,13 +1572,14 @@ def doctest_ACLView():
                   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
         <html>
         ...
-        <form method="POST" action="http://127.0.0.1">
-          <h3>
-              Access control
-          </h3>
+        <form method="post" action="http://127.0.0.1">
         ...
+          <h3>
+              Access control for SchoolBell
+          </h3>
           <fieldset>
             <legend>Permissions for Groups</legend>
+        ...
             <table class="acl">
               <tr class="header">
                  <th class="principal">Group</th>
@@ -1070,7 +1593,7 @@ def doctest_ACLView():
                  <th class="permission">Manage membership</th>
               </tr>
         ...
-              <tr class="even">
+              <tr class="odd">
                  <th class="principal">
                     office
                     <input type="hidden" value="1"
@@ -1176,16 +1699,18 @@ def doctest_ACLView():
         >>> grants.getPermissionsForPrincipal('sb.group.3')
         [('schoolbell.create', PermissionSetting: Allow)]
 
-        >>> pprint(view.persons)
+        >>> pprint(list(view.batches['persons']))
         [{'id': u'sb.person.albert',
           'perms': ['schoolbell.view', 'schoolbell.edit'],
           'title': 'Albert'},
          {'id': u'sb.person.marius',
           'perms': ['schoolbell.create'],
           'title': 'Marius'}]
-        >>> pprint(view.groups)
-        [{'perms': [], 'id': 'zope.authenticated', 'title': 'Authenticated users'},
-         {'perms': [], 'id': 'zope.unauthenticated', 'title': 'Unauthenticated users'},
+        >>> pprint(list(view.batches['groups']))
+        [{'perms': [], 'id': 'zope.authenticated', 'title': u'Authenticated users'},
+         {'id': 'zope.unauthenticated',
+          'perms': [],
+          'title': u'Unauthenticated users'},
          {'perms': ['schoolbell.create'], 'id': u'sb.group.3', 'title': 'office'},
          {'perms': [], 'id': u'sb.group.4', 'title': 'mgmt'}]
 
@@ -1206,7 +1731,7 @@ def doctest_ACLView():
                   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
         <html>
         ...
-              <tr class="even">
+              <tr class="odd">
                  <th class="principal">
                     office
                     <input type="hidden" value="1"
@@ -1413,10 +1938,11 @@ def doctest_ACLView_inheritance():
         >>> View = SimpleViewClass("../templates/acl.pt", bases=(ACLView, ))
         >>> request = TestRequest()
         >>> view = View(app['persons'], request)
+        >>> view.update()
 
-    Now, view.persons shows the principals have the permissions:
+    Now, view.batches['persons'] shows the principals have the permissions:
 
-        >>> pprint(view.persons)
+        >>> pprint(list(view.batches['persons']))
         [{'id': u'sb.person.albert',
           'perms': ['schoolbell.controlAccess'],
           'title': 'Albert'},
@@ -1460,10 +1986,10 @@ def doctest_ACLView_inheritance():
 
     """
 
-def doctest_hasPermission():
+def doctest_hasPermissions():
     r"""The Zope security machinery does not have tools to check
     whether a random principal has some permission on some object.  So
-    we need co construct our own.
+    we need to construct our own.
 
     Set up for local grants:
 
@@ -1500,42 +2026,94 @@ def doctest_hasPermission():
         >>> from schoolbell.app.app import Person
         >>> app['persons']['1'] = Person('joe', title='Joe')
 
-    He does not have a 'super' permission on our schoolbell app:
+    He does not have neither 'super' nor 'duper' permissions on our
+    schoolbell app:
 
-        >>> from schoolbell.app.browser.app import hasPermission
-        >>> hasPermission('super', app, 'sb.person.joe')
-        False
+        >>> from schoolbell.app.browser.app import hasPermissions
+        >>> hasPermissions(['super', 'duper'], app, 'sb.person.joe')
+        [False, False]
 
     However, we can add a local grant:
 
         >>> perms = IPrincipalPermissionManager(app)
         >>> perms.grantPermissionToPrincipal('super', 'sb.person.joe')
 
-    Now, hasPermission returns true:
+    And everything changes!
 
-        >>> hasPermission('super', app, 'sb.person.joe')
-        True
+        >>> hasPermissions(['super', 'duper'], app, 'sb.person.joe')
+        [True, False]
 
     The same works for subobjects:
 
-        >>> hasPermission('super', app['persons'], 'sb.person.joe')
-        True
-        >>> hasPermission('super', app['persons']['joe'], 'sb.person.joe')
-        True
+        >>> hasPermissions(['duper', 'super'], app['persons'], 'sb.person.joe')
+        [False, True]
+        >>> hasPermissions(['super', 'duper'], app['persons']['joe'], 'sb.person.joe')
+        [True, False]
 
     Also, it works gracefully for None or random objects:
 
-        >>> hasPermission('super', None, 'sb.person.joe')
-        False
-        >>> hasPermission('super', object(), 'sb.person.joe')
-        False
+        >>> hasPermissions(['super'], None, 'sb.person.joe')
+        [False]
+        >>> hasPermissions(['super'], object(), 'sb.person.joe')
+        [False]
+
     """
 
+def doctest_ApplicationPreferencesView():
+    """
+
+    We need to setup a SchoolBellApplication site and build our
+    ISchoolBellApplication adapter:
+
+        >>> from schoolbell.app.app import SchoolBellApplication, Person
+        >>> from zope.app.component.site import LocalSiteManager
+        >>> app = SchoolBellApplication()
+        >>> app.setSiteManager(LocalSiteManager(app))
+        >>> from zope.app.component.hooks import setSite
+        >>> setSite(app)
+        >>> from schoolbell.app.browser.app import ApplicationPreferencesView
+        >>> from schoolbell.app.app import getApplicationPreferences
+        >>> from zope.app.annotation.interfaces import IAnnotations
+        >>> from schoolbell.app.interfaces import IApplicationPreferences
+        >>> from schoolbell.app.interfaces import ISchoolBellApplication
+        >>> from schoolbell.app.app import SchoolBellApplication
+
+        >>> setup.setUpAnnotations()
+        >>> ztapi.provideAdapter(ISchoolBellApplication,
+        ...                      IApplicationPreferences,
+        ...                      getApplicationPreferences)
+        >>> from schoolbell.app.app import getSchoolBellApplication
+
+
+    Make sure we can create a view:
+
+        >>> app = getSchoolBellApplication()
+
+        >>> request = TestRequest()
+
+        >>> view = ApplicationPreferencesView(app, request)
+
+    Now we can setup a post and set the site title:
+
+        >>> request = TestRequest(form={'UPDATE_SUBMIT': 'Update',
+        ...                             'field.title': 'Company Calendars',})
+        >>> view = ApplicationPreferencesView(app, request)
+
+        >>> view.update()
+
+        >>> prefs = getApplicationPreferences(app)
+        >>> prefs.title
+        u'Company Calendars'
+
+    """
+
+
 def test_suite():
+    optionflags = (doctest.ELLIPSIS | doctest.REPORT_NDIFF
+                   | doctest.REPORT_ONLY_FIRST_FAILURE)
     suite = unittest.TestSuite()
     suite.addTest(doctest.DocTestSuite(setUp=setUp, tearDown=tearDown,
-                                       optionflags=doctest.ELLIPSIS|
-                                                   doctest.REPORT_NDIFF))
+                                       optionflags=optionflags))
     return suite
 
 
