@@ -20,10 +20,11 @@ r"""
 iCalendar parsing and generating.
 
 iCalendar (RFC 2445) is a big and hard-to-read specification.  This module
-supports only a subset of it: VEVENT components with a limited set of
-attributes and a limited recurrence model.  The subset should be sufficient
-for interoperation with desktop calendaring applications like Apple's iCal,
-Mozilla Calendar, Evolution and KOrganizer.
+supports only a subset of it: VEVENT and VFREEBUSY components with a limited
+set of attributes and a limited recurrence model.  The subset should be
+sufficient for interoperation with desktop calendaring applications like
+Apple's iCal, Mozilla Calendar, Evolution and KOrganizer as well as the
+Outlook meeting planner.
 
 If you have a calendar, you can convert it to an iCalendar file like this:
 
@@ -64,7 +65,7 @@ There is some trickery to make empty calendars work:
     []
 
 
-$Id: icalendar.py 3049 2005-02-28 13:13:28Z gintas $
+$Id: icalendar.py 4427 2005-08-01 17:36:39Z bskahan $
 """
 
 import datetime
@@ -77,6 +78,26 @@ from schoolbell.calendar.simple import SimpleCalendarEvent
 
 EMPTY_CALENDAR_PLACEHOLDER = 'empty-calendar-placeholder@schooltool.org'
 
+def convert_event_to_vfb(event):
+    r"""Convert an ICalendarEvent to iCalendar FREEBUSY entry.
+
+    Returns a list of strings (without newlines) in UTF-8.
+
+        >>> from datetime import datetime, timedelta
+        >>> event = SimpleCalendarEvent(datetime(2004, 12, 16, 10, 7, 29),
+        ...                             timedelta(hours=1), "iCal rendering",
+        ...                             location="Big room",
+        ...                             description="Blah blah\nblah!",
+        ...                             unique_id="12345678-5432@example.com")
+        >>> lines = convert_event_to_vfb(event)
+        >>> print "\n".join(lines)
+        FREEBUSY:20041216T100729/20041216T11072900Z
+
+    """
+    # XXX: not handling recurrence yet
+    return ["FREEBUSY:%s/%s00Z" % (ical_datetime(event.dtstart),
+                               ical_datetime(event.dtstart + event.duration))]
+
 
 def convert_event_to_ical(event):
     r"""Convert an ICalendarEvent to iCalendar VEVENT component.
@@ -96,7 +117,7 @@ def convert_event_to_ical(event):
         SUMMARY:iCal rendering
         LOCATION:Big room
         DESCRIPTION:Blah blah\nblah!
-        DTSTART:20041216T100729
+        DTSTART:20041216T100729Z
         DURATION:PT1H
         DTSTAMP:...
         END:VEVENT
@@ -112,11 +133,31 @@ def convert_event_to_ical(event):
         UID:12345678-9876@example.com
         SUMMARY:iCal tests
         RRULE:FREQ=DAILY;INTERVAL=1
-        DTSTART:20050211T224250
+        DTSTART:20050211T224250Z
         DURATION:PT15M
         DTSTAMP:...
         END:VEVENT
 
+    All-day events have DTSTART as a date:
+
+        >>> from schoolbell.calendar.recurrent import WeeklyRecurrenceRule
+        >>> from schoolbell.calendar.recurrent import DailyRecurrenceRule
+        >>> event = SimpleCalendarEvent(datetime(2005, 2, 11, 0, 0, 0),
+        ...                             timedelta(days=2), "iCal tests",
+        ...                             recurrence=WeeklyRecurrenceRule(),
+        ...                             unique_id="12345678-9876@example.com",
+        ...                             allday=True)
+        >>> lines = convert_event_to_ical(event)
+        >>> print "\n".join(lines)
+        BEGIN:VEVENT
+        UID:12345678-9876@example.com
+        SUMMARY:iCal tests
+        RRULE:FREQ=WEEKLY;BYDAY=FR;INTERVAL=1
+        DTSTART;VALUE=DATE:20050211
+        DURATION:P2D
+        DTSTAMP:...
+        END:VEVENT
+
     """
     dtstamp = datetime.datetime.utcnow().strftime("%Y%m%dT%H%M%SZ")
     result = [
@@ -130,15 +171,74 @@ def convert_event_to_ical(event):
     if event.recurrence is not None:
         start = event.dtstart
         result.extend(event.recurrence.iCalRepresentation(start))
-    result += [
-        "DTSTART:%s" % ical_datetime(event.dtstart),
-        "DURATION:%s" % ical_duration(event.duration),
-        "DTSTAMP:%s" % dtstamp,
-        "END:VEVENT",
-    ]
+
+    if event.allday:
+        dtstart = 'DTSTART;VALUE=DATE:%s' % ical_date(event.dtstart)
+    else:
+        dtstart = 'DTSTART:%sZ' % ical_datetime(event.dtstart)
+    result += [dtstart,
+               "DURATION:%s" % ical_duration(event.duration),
+               "DTSTAMP:%s" % dtstamp,
+               "END:VEVENT"]
     return result
 
 
+def convert_calendar_to_vfb(calendar):
+    r"""Convert an ICalendar to a VFREEBUSY component
+    Returns a list of strings (without newlines) in UTF-8.  They should be
+    joined with '\r\n' to get a valid iCalendar file.
+
+        >>> from schoolbell.calendar.simple import ImmutableCalendar
+        >>> from schoolbell.calendar.simple import SimpleCalendarEvent
+        >>> from datetime import datetime, timedelta
+        >>> event1 = SimpleCalendarEvent(datetime(2004, 11, 16, 10, 7, 29),
+        ...                             timedelta(hours=1), "iCal rendering",
+        ...                             location="Big room",
+        ...                             unique_id="12345678-5432@example.com")
+        >>> event2 = SimpleCalendarEvent(datetime(2004, 12, 16, 10, 7, 29),
+        ...                             timedelta(hours=2), "iCal rendering",
+        ...                             location="Big room",
+        ...                             unique_id="12345678-9876@example.com")
+        >>> calendar = ImmutableCalendar([event1, event2])
+        >>> lines = convert_calendar_to_vfb(calendar)
+        >>> print "\n".join(lines)
+        BEGIN:VCALENDAR
+        VERSION:2.0
+        PRODID:-//SchoolTool.org/NONSGML SchoolBell//EN
+        METHOD:PUBLISH
+        BEGIN:VFREEBUSY
+        FREEBUSY:20041116T100729/20041116T11072900Z
+        FREEBUSY:20041216T100729/20041216T12072900Z
+        END:VFREEBUSY
+        END:VCALENDAR
+
+    XXX VFREEBUSY should just expand every occurence
+
+        >>> lines = convert_calendar_to_vfb(ImmutableCalendar())
+        >>> print "\n".join(lines)
+        BEGIN:VCALENDAR
+        VERSION:2.0
+        PRODID:-//SchoolTool.org/NONSGML SchoolBell//EN
+        METHOD:PUBLISH
+        BEGIN:VFREEBUSY
+        END:VFREEBUSY
+        END:VCALENDAR
+
+
+    """
+    header = ["BEGIN:VCALENDAR",
+              "VERSION:2.0",
+              "PRODID:-//SchoolTool.org/NONSGML SchoolBell//EN",
+              "METHOD:PUBLISH",
+              "BEGIN:VFREEBUSY"]
+    events = []
+    for event in calendar:
+        events += convert_event_to_vfb(event)
+    footer = ["END:VFREEBUSY",
+              "END:VCALENDAR"]
+    return header + events + footer
+
+
 def convert_calendar_to_ical(calendar):
     r"""Convert an ICalendar to iCalendar VCALENDAR component.
 
@@ -162,7 +262,7 @@ def convert_calendar_to_ical(calendar):
         UID:12345678-5432@example.com
         SUMMARY:iCal rendering
         LOCATION:Big room
-        DTSTART:20041216T100729
+        DTSTART:20041216T100729Z
         DURATION:PT1H
         DTSTAMP:...
         END:VEVENT
@@ -179,21 +279,17 @@ def convert_calendar_to_ical(calendar):
         BEGIN:VEVENT
         UID:...
         SUMMARY:Empty calendar
-        DTSTART:19700101T000000
+        DTSTART:19700101T000000Z
         DURATION:P0D
         DTSTAMP:...
         END:VEVENT
         END:VCALENDAR
 
     """
-    header = [
-        "BEGIN:VCALENDAR",
-        "VERSION:2.0",
-        "PRODID:-//SchoolTool.org/NONSGML SchoolBell//EN",
-    ]
-    footer = [
-        "END:VCALENDAR"
-    ]
+    header = ["BEGIN:VCALENDAR",
+              "VERSION:2.0",
+              "PRODID:-//SchoolTool.org/NONSGML SchoolBell//EN"]
+    footer = ["END:VCALENDAR"]
     events = []
     for event in calendar:
         events += convert_event_to_ical(event)
@@ -296,7 +392,7 @@ def ical_duration(value):
         return "%sP%dD%s" % (sign, abs(value.days), timepart)
 
 
-def read_icalendar(icalendar_text):
+def read_icalendar(icalendar_text, charset='UTF-8'):
     """Read an iCalendar file and return calendar events.
 
     Returns an iterator over calendar events.
@@ -310,13 +406,14 @@ def read_icalendar(icalendar_text):
     if not hasattr(icalendar_text, 'read'):
         # It is not a file-like object -- let's assume it is a string
         icalendar_text = StringIO(icalendar_text)
-    reader = ICalReader(icalendar_text)
+    reader = ICalReader(icalendar_text, charset)
     for vevent in reader.iterEvents():
         if vevent.uid == EMPTY_CALENDAR_PLACEHOLDER:
             continue # Ignore empty calendar placeholder "event"
 
         # Currently SchoolBell does not support all-day events, so we must
         # convert them into ordinary events that last 24 hours
+        # TODO: Now it does and this should be fixed!
         dtstart = vevent.dtstart
         if not isinstance(dtstart, datetime.datetime):
             dtstart = datetime.datetime.combine(dtstart,
@@ -481,6 +578,8 @@ class ICalReader:
         """
         record = []
         for line in self.file:
+            if not line.strip():
+                continue
             if line[0] in '\t ':
                 line = line[1:]
             elif record:
@@ -625,12 +724,21 @@ def parse_date_time(value):
     >>> parse_date_time('20030405T060708')
     datetime.datetime(2003, 4, 5, 6, 7, 8)
 
+    >>> parse_date_time('20030405T060708Z')
+    datetime.datetime(2003, 4, 5, 6, 7, 8)
+
     Examples of invalid arguments:
 
+    >>> parse_date_time('20030405T060708+05:00')
+    Traceback (most recent call last):
+      ...
+    ValueError: Invalid iCalendar date-time: '20030405T060708+05:00'
+
     >>> parse_date_time('20030405T060708A')
     Traceback (most recent call last):
       ...
     ValueError: Invalid iCalendar date-time: '20030405T060708A'
+
     >>> parse_date_time('')
     Traceback (most recent call last):
       ...
@@ -648,12 +756,8 @@ def parse_date_time(value):
     dt = datetime.datetime(int(y), int(m), int(d),
                            int(hh), int(mm), int(ss))
     if utc:
-        # In the future we might want to get the timezone from the iCalendar
-        # file, but for now using the local timezone of the server should
-        # be adequate.
-        timetuple = dt.timetuple()
-        ticks = calendar.timegm(timetuple)
-        dt = datetime.datetime.fromtimestamp(ticks)
+        # currently we only handle iCal files in UTC.
+        pass
 
     return dt
 
@@ -828,9 +932,9 @@ def parse_recurrence_rule(value):
     An example that includes use of UNTIL:
 
     >>> parse_recurrence_rule('FREQ=DAILY;UNTIL=20041008T000000')
-    DailyRecurrenceRule(1, None, datetime.datetime(2004, 10, 8, 0, 0), ())
+    DailyRecurrenceRule(1, None, datetime.date(2004, 10, 8), ())
     >>> parse_recurrence_rule('FREQ=DAILY;UNTIL=20041008')
-    DailyRecurrenceRule(1, None, datetime.datetime(2004, 10, 8, 0, 0), ())
+    DailyRecurrenceRule(1, None, datetime.date(2004, 10, 8), ())
 
     Of course, other recurrence frequencies may be used:
 
@@ -1065,10 +1169,8 @@ class VEvent:
 
         self.rrule = self.getOne('RRULE', None)
         if self.rrule is not None and self.exdates:
-            if self._getType('EXDATE') == 'DATE-TIME':
-                exceptions = [dt.date() for dt in self.exdates]
-            else:
-                exceptions = self.exdates
+            exceptions = [datetime.date(dt.year, dt.month, dt.day)
+                          for dt in self.exdates]
             self.rrule = self.rrule.replace(exceptions=exceptions)
 
     def _extractListOfDates(self, key, accepted_types, all_day_event):
@@ -1093,7 +1195,10 @@ class VEvent:
         return dates
 
     def _getType(self, property):
-        """Return the type of the property value."""
+        """Return the type of the property value.
+
+        Only call getType for properties that do not occur more than once.
+        """
         key = property.upper()
         values = self._props[key]
         assert len(values) == 1
