@@ -19,7 +19,7 @@
 """
 Recurrence rules.
 
-$Id: recurrent.py 3316 2005-03-14 23:14:22Z jinty $
+$Id: recurrent.py 5021 2005-09-20 09:24:55Z mg $
 """
 
 import datetime
@@ -55,7 +55,10 @@ class RecurrenceRule(object):
     def __init__(self, interval=1, count=None, until=None, exceptions=()):
         self._interval = interval
         self._count = count
-        self._until = until
+        if until and isinstance(until, datetime.datetime):
+            self._until = until.date()
+        else:
+            self._until = until
         self._exceptions = tuple(exceptions) # XXX I'd prefer a set.
         self._validate()
 
@@ -112,20 +115,23 @@ class RecurrenceRule(object):
         """
         return hash(self._tupleForComparison())
 
-    def apply(self, event, enddate=None):
+    def apply(self, event, startdate=None, enddate=None):
         """Generator that generates dates of recurrences"""
         if enddate:
             assert isinstance(enddate, datetime.date), "enddate must be a date"
             assert not isinstance(enddate, datetime.datetime), \
                     "enddate must be a date, not a datetime"
-        cur = event.dtstart.date()
-        count = 0
+        if startdate is None:
+            startdate = event.dtstart.date()
+
+        count, cur = self._scroll(event, startdate)
+
         while True:
             if ((enddate and cur > enddate) or
                 (self.count is not None and count >= self.count) or
                 (self.until and cur > self.until)):
                 break
-            if cur not in self.exceptions:
+            if cur not in self.exceptions and cur >= startdate:
                 yield cur
             count += 1
             cur = self._nextRecurrence(cur)
@@ -178,6 +184,17 @@ class DailyRecurrenceRule(RecurrenceRule):
 
     ical_freq = 'DAILY'
 
+    def _scroll(self, event, startdate):
+        """Given a startdate, finds a nearest event recurrence and its nr"""
+        if startdate >= event.dtstart.date():
+            days = (startdate - event.dtstart.date()).days
+        else:
+            days = 0
+        count = days / self.interval
+        cur = event.dtstart.date() + datetime.timedelta(count * self.interval)
+        return count, cur
+
+
 
 class YearlyRecurrenceRule(RecurrenceRule):
     """Yearly recurrence rule.
@@ -190,8 +207,29 @@ class YearlyRecurrenceRule(RecurrenceRule):
 
     def _nextRecurrence(self, date):
         """Adds the basic step of recurrence to the date"""
-        nextyear = date.year + self.interval
-        return date.replace(year=nextyear)
+        delta = self.interval
+        while True:
+            try:
+                nextyear = date.year + delta
+                return date.replace(year=nextyear)
+            except ValueError:
+                # Jump over illegal leap days
+                delta += self.interval
+
+    def _scroll(self, event, startdate):
+        """Given a startdate, finds a nearest event recurrence and its nr"""
+        if startdate >= event.dtstart.date():
+            years = startdate.year - event.dtstart.date().year
+        else:
+            years = 0
+        count = years / self.interval
+        while True:
+            try:
+                newyear = event.dtstart.date().year + count * self.interval
+                cur = event.dtstart.date().replace(year=newyear)
+                return count, cur
+            except ValueError:
+                count -= self.interval
 
     def _iCalArgs(self, dtstart):
         """Return iCalendar parameters specific to monthly reccurence."""
@@ -212,7 +250,10 @@ class WeeklyRecurrenceRule(RecurrenceRule):
                  weekdays=()):
         self._interval = interval
         self._count = count
-        self._until = until
+        if until and isinstance(until, datetime.datetime):
+            self._until = until.date()
+        else:
+            self._until = until
         self._exceptions = tuple(exceptions)
         self._weekdays = tuple(weekdays)
         self._validate()
@@ -247,14 +288,27 @@ class WeeklyRecurrenceRule(RecurrenceRule):
         return (self.__class__.__name__, self.interval, self.count,
                 self.until, self.exceptions, self.weekdays)
 
-    def apply(self, event, enddate=None):
+    def _scroll(self, event, startdate):
+        """Fast-forward to recurrences near startdate"""
+        start = event.dtstart.date()
+        weeks = weekspan(start, startdate)
+        perweek = len(self.weekdays) + 1 # nr of recurrences per week
+        count = weeks / self.interval * perweek
+        if count > perweek:
+            count -= perweek # to get entries with weekday < that of start
+        offset = datetime.timedelta(7) * count / perweek * self.interval
+        return count, start + offset
+
+    def apply(self, event, startdate=None, enddate=None):
         """Generate dates of recurrences."""
         if enddate:
             assert isinstance(enddate, datetime.date), "enddate must be a date"
             assert not isinstance(enddate, datetime.datetime), \
                     "enddate must be a date, not a datetime"
-        cur = start = event.dtstart.date()
-        count = 0
+        start = event.dtstart.date()
+        if startdate is None:
+            startdate = start
+        count, cur = self._scroll(event, startdate)
         weekdays = Set(self.weekdays)
         weekdays.add(event.dtstart.weekday())
         while True:
@@ -266,7 +320,7 @@ class WeeklyRecurrenceRule(RecurrenceRule):
             # the desired weekday
             if (weekspan(start, cur) % self.interval == 0 and
                 cur.weekday() in weekdays):
-                if cur not in self.exceptions:
+                if cur not in self.exceptions and cur >= startdate:
                     yield cur
                 count += 1
             cur = self._nextRecurrence(cur)
@@ -333,6 +387,26 @@ class MonthlyRecurrenceRule(RecurrenceRule):
         return (self.__class__.__name__, self.interval, self.count,
                 self.until, self.exceptions, self.monthly)
 
+    def _scroll(self, event, startdate):
+        """Given a startdate, finds a nearest event recurrence and its nr"""
+        dstart = event.dtstart.date()
+        if startdate > event.dtstart.date():
+            months = (12 * (startdate.year - dstart.year) +
+                      startdate.month - dstart.month)
+        else:
+            months = 0
+
+        count = months / self.interval
+        while True:
+            try:
+                year, month = divmod(dstart.year * 12 + dstart.month +
+                                     count * self.interval - 1, 12)
+                month += 1 # 1..12
+                cur = event.dtstart.date().replace(year=year, month=month)
+                return count, cur
+            except ValueError:
+                count -= 1
+
     def _nextRecurrence(self, date):
         """Add basic step of recurrence to the date."""
         year = date.year
@@ -345,60 +419,68 @@ class MonthlyRecurrenceRule(RecurrenceRule):
             except ValueError:
                 continue
 
-    def apply(self, event, enddate=None):
+    def apply(self, event, startdate=None, enddate=None):
         if enddate:
             assert isinstance(enddate, datetime.date), "enddate must be a date"
             assert not isinstance(enddate, datetime.datetime), \
                     "enddate must be a date, not a datetime"
         if self.monthly == 'monthday':
-            for date in  RecurrenceRule.apply(self, event, enddate):
+            for date in  RecurrenceRule.apply(self, event, startdate, enddate):
                 yield date
         elif self.monthly == 'weekday':
-            for date in self._applyWeekday(event, enddate):
+            for date in self._applyWeekday(event, startdate, enddate):
                 yield date
         elif self.monthly == 'lastweekday':
-            for date in self._applyLastWeekday(event, enddate):
+            for date in self._applyLastWeekday(event, startdate, enddate):
                 yield date
 
-    def _applyWeekday(self, event, enddate=None):
+    def _applyWeekday(self, event, startdate=None, enddate=None):
         """Generator that generates dates of recurrences."""
-        cur = start = event.dtstart.date()
-        count = 0
-        year = start.year
-        month = start.month
+        start = event.dtstart.date()
         weekday = start.weekday()
         index = start.day / 7 + 1
 
+        if startdate is None:
+            startdate = start
+
+        count, cur = self._scroll(event, startdate)
+        year = cur.year
+        month = cur.month
+
         while True:
             cur = monthindex(year, month, index, weekday)
             if ((enddate and cur > enddate) or
                 (self.count is not None and count >= self.count) or
                 (self.until and cur > self.until)):
                 break
-            if cur not in self.exceptions:
+            if cur not in self.exceptions and cur >= startdate:
                 yield cur
             count += 1
             # Next month, please.
             year, month = divmod(year * 12 + month + self.interval - 1, 12)
             month += 1
 
-    def _applyLastWeekday(self, event, enddate=None):
+    def _applyLastWeekday(self, event, startdate=None, enddate=None):
         """Generator that generates dates of recurrences."""
-        cur = start = event.dtstart.date()
-        count = 0
-        year = start.year
-        month = start.month
+        start = event.dtstart.date()
         weekday = start.weekday()
-        daysinmonth = calendar.monthrange(year, month)[1]
+        daysinmonth = calendar.monthrange(start.year, start.month)[1]
         index = (start.day - daysinmonth - 1) / 7
 
+        if startdate is None:
+            startdate = start
+
+        count, cur = self._scroll(event, startdate)
+        year = cur.year
+        month = cur.month
+
         while True:
             cur = monthindex(year, month, index, weekday)
             if ((enddate and cur > enddate) or
                 (self.count is not None and count >= self.count) or
                 (self.until and cur > self.until)):
                 break
-            if cur not in self.exceptions:
+            if cur not in self.exceptions and cur >= startdate:
                 yield cur
             count += 1
             # Next month, please.
